//! This javascript is executed inside the iframe of a dcomponent item (dcitem: Message, Article...)
//! It manages : 
//! - cycling between highlighted days and keywords
//! - communication with parent
//! 
//! It cannot use jQuery.
//! Communication with iframe is done using messages
//! The iframe uses the "sandbox" property, so its origin is null.

// namespace
(function() {
window.dcitem_inside_frame = window.dcitem_inside_frame || {};
var ns=window.dcitem_inside_frame;// shortcut


// ***************** 

window.addEventListener("message", 
						dcitem_inside_frame_receive_message, false);

window.addEventListener("load", function()
						{
							add_popup_event_to_highlighted_dates();
						},true);

// ***************** 


// Receives prev and next hilight cycle messages from parent window.
// (this is also used to reply to "getheight" message).
function dcitem_inside_frame_receive_message(e)
{
	//console.log("inside iframe: received message:",e.origin,e.data,e);

	// Check origin. This is NOT for security (we are running inside untrusted frame, so its JS can change everything)
	// It is just usefull to avoid confusing with messages that might be generated by other sources included in this frame.
	var std_base_host=demosphere_config.std_base_url.replace(/^(https?:\/\/[^\/]*)\/.*$/,'$1');
	if(e.origin.replace(/^https:/,'http:') != std_base_host.replace(/^https:/,'http:')){return;}

	// determine and execute command
	var cmd=e.data.match(/^([a-z]+)(:(.*))?$/);
	if(cmd!==null){cmd=cmd[1];}
	switch(cmd)
	{
	case 'getheight':
		// respond to getheight by sending this document's height
		e.source.postMessage("dcitem-height:"+ e.data.match(/^([a-z]+)(:(.*))?$/)[3] + 
							 ":" +document.body.offsetHeight,e.origin);  
		break;
	case "next":
	case "prev":
		dcitem_inside_frame_highlight_cycle(cmd);
		break;
	default:
		// ignore invalid commands
		return;
	}
}

//! Scroll window to next or previous highlighted element
function dcitem_inside_frame_highlight_cycle(cycleDir)
{
	var data;
	if(!window.dcitemInsideFrameCycleData)
	{
		window.dcitemInsideFrameCycleData={};
		window.dcitemInsideFrameCycleData.elList=
			dcitem_inside_frame_highlight_el_list(document);
	}
	data=window.dcitemInsideFrameCycleData;
	var list=data.elList;
	var l=list.length;
	if(l==0){return;}
	// Find the current index (or init if first time)
	var prevIndex=false;
	var ypos=0;
	var isFirst=false;
	if(data.currentIndex==undefined){data.currentIndex=cycleDir=='prev' ? 0:l-1;isFirst=true;}
	else{prevIndex=data.currentIndex;ypos=list[prevIndex].ypos;}
	var index=data.currentIndex;

	// Find the next/prev index. In a simple case this is just index+-1.
	// But we may want to ignore elements that are too close (ypos).
	for(var maxAbort=0;maxAbort<l;maxAbort++)
	{
		if(!isFirst && ((cycleDir==='next' && index==l-1) ||
						(cycleDir==='prev' && index==0  )))
		{dcitem_inside_frame_highlight_cycle_flash();}
		index=(index+(cycleDir==='next' ? 1 : -1)+l)%l;
		// check if the type of element is ok
		//if(type of diff ok){break;} FIXME
		if(Math.abs(list[index].ypos-ypos)<8){continue;}
		break;
	}
	//console.log("index1:"+index+"\n");
	
	// now scroll the container to the new y position
	var yscroll=list[index].ypos-25;// add a margin so some preceding content is visible
	window.scroll(0,yscroll);

	// highlight the current element, unhighlight the previously highlighted one
	list[index].el.className+=" demosphere_active";
	if(prevIndex!==false)
	{
		list[prevIndex].el.className=list[prevIndex].el.className.replace(' demosphere_active','');
	}
	// remember the current index
	data.currentIndex=index;
}

//! Returns a list of all highlighted elements by matching a list of classnames.
//! The list also contains y-positions of each element and is sorted by y-pos.
function dcitem_inside_frame_highlight_el_list(parent)
{
	var classes=["demosphere_highlight_diff","demosphere_timestamp","demosphere_keyword"];
	var list=[];
	for(var i=0;i<classes.length;i++)
	{
		var classEls=parent.getElementsByClassName(classes[i]);
		for(var j=0,ll=classEls.length;j<ll;j++)
		{
			list.push({el:classEls[j],ypos:dcitem_inside_frame_find_pos_y(classEls[j])});
		}
	}
	list.sort(function(a,b){return a.ypos-b.ypos;});
	return list;
}

function dcitem_inside_frame_highlight_cycle_flash()
{
	var waitDiv=document.createElement("div");
	document.body.appendChild(waitDiv);
	waitDiv.id="demosWaitDiv";
	waitDiv.style.position="fixed";
	waitDiv.style.width="100%";
	waitDiv.style.height="300px";
	waitDiv.style.top="0px";
	waitDiv.style.left="0px";
	waitDiv.style.backgroundColor="green";
	waitDiv.style.zIndex="100000";
	//alert("falsh");
	setTimeout(function (){waitDiv.parentNode.removeChild(waitDiv);},200);
}


//*********************************

// cross browser: get absolute pos of an object
// http://www.quirksmode.org
function dcitem_inside_frame_find_pos_x(obj)
{
	var curleft = 0;
	if (obj.offsetParent)
	{
		while (obj.offsetParent)
		{
			curleft += obj.offsetLeft
			obj = obj.offsetParent;
		}
	}
	else if (obj.x)
		curleft += obj.x;
	return curleft;
}

// cross browser: get absolute pos of an object
// http://www.quirksmode.org
function dcitem_inside_frame_find_pos_y(obj)
{
	var curtop = 0;
	if (obj.offsetParent)
	{
		while (obj.offsetParent)
		{
			curtop += obj.offsetTop
			obj = obj.offsetParent;
		}
	}
	else if (obj.y)
		curtop += obj.y;
	return curtop;
}

// Exports:
//window.dcitem_inside_frame.abcd=abcd;

// end namespace wrapper
}());
