<?php

/**
 * This is the main form that moderators use to edit events.
 * It can also be displayed to less priviledged users (frontpage event creator, selfEdit...)
 * This form includes a Place form generated by demosphere_place_form()
 * Many elements of this form are created by using form_dbobject for Event.
 * 
 */
function demosphere_event_edit_form($id=false)
{
	global $demosphere_config,$base_url,$user,$currentPage;
	require_once 'dlib/form.php';
	require_once 'demosphere-common.php';
	require_once 'demosphere-event-repetition.php';
	require_once 'demosphere-misc.php';
	require_once 'demosphere-place.php';
	require_once 'demosphere-date-time.php';
	require_once 'demosphere-self-edit.php';

	// User wants to create a new empty event.
	if($id===false){$event=new Event();$event->startTime=0;}
	else           
	{
		// User wants to edit an existing event.
		$event=Event::fetch($id,false);
		if($event===null){dlib_not_found_404();}
	}

	// permission/access managed here, not in demosphere_event_access() because of complex special case
	if(!$event->access('edit'))
	{
		// very special case: user can self edit group, but not this event (for example: event is too old)
		if(demosphere_self_edit_check_group($event->repetitionGroupId))
		{
			dlib_redirect($base_url.'/repetition-group/'.$event->repetitionGroupId.'/confirm');
		}
		dlib_permission_denied_403();
	}


	// Think about this (self edit user might be confused by tab switching between fullwidth and not fullwidth)
	$currentPage->bodyClasses[]="fullWidth";

	// add all css and java for this form
	demosphere_event_edit_form_css_js();

	// Tabs : (view, edit, repetitions...)
	require_once 'demosphere-event.php';
	demosphere_event_tabs($event,'edit');

	// Create standard form items for Event object.
	// We will only copy the ones we want to $form.
	list($objForm,$formOpts)=form_dbobject($event);
	//var_dump(array_keys($objForm));

	// The event object, so it can be used in submit handler
	$form['form_object']=$objForm['form_object'];

	// non admin get extra help message to make sure they notice the help button
	if(!$user->checkRoles('admin','moderator'))
	{
		// extra submit button (some users dont see bottom button)
		$form['non-admin-submit'] =   
			[
				'type' => 'submit',
				'value' => t('Save'),
			];
		$form['non-admin-help-message'] = 
			['html'=>'<p id="non-admin-help-message">'.
			 t("For help on using this form, !button click here</span> and then click on a blue question mark. Don't forget to press the save button when you are over.",['!button'=>'<span class="demosphere-help-extra-button" onmousedown="dhelp(event,\'help\')"   >']).'</p>',
			];
	}

	// if this is selfEdit, set selfEdit last accessed time, and set value in form for later use 
	$form['self-edit-ok']=['type'=>'data','value'=>false];
	if($event->id && !$user->checkRoles('admin','moderator') && $event->canSelfEdit())
	{
		demosphere_backup_site_assert();
		$form['self-edit-ok']['value']=true;
	}

	// ********* repetitions

	$isRepetition=$event->id && count($event->getAllRepetitions())>1;

	if($isRepetition)
	{
		$form['repetition-head'] =
			[ 'html'=>'<p id="repetition-head">'.
			  t('This event is part of a group of repetitions.').'</p>'];
	}

	// ********* title

	$title=$event->title;
	if(isset($event->htmlTitle) && $event->htmlTitle!==null && 
	   $event->htmlTitle!=='')
	{
		$title=$event->htmlTitle;
		$title=str_replace( '<strong>','_',$title);
		$title=str_replace('</strong>','_',$title);
		$title=html_entity_decode($title,ENT_QUOTES,'UTF-8');
	}

	$form['title']=$objForm['title'];
	$form['title']['title']=t('Title');
	$form['title']['required']=true;
	$form['title']['default-value']=$title;
	$form['title']['attributes']=['size' => 60,
								  'maxlength' => 128,
								  'spellcheck' => 'true'
								 ];
	$form['title']['field-prefix']='<span id="title-field-wrap">';
	$form['title']['field-suffix']='<span id="edit-title-warn" style="display:none">'.t('Title too long!').'<span></span></span>'.
		'<span id="title-buttons">'.
		'<button type="button" id="title-bold"     title="'.t('Important word').'"><span></span></button>'.
		'<button type="button" id="title-downcase" title="'.t('Lower case'    ).'"><span></span></button>'.
		'<button type="button" id="title-upcase"   title="'.t('Upper case'    ).'"><span></span></button>'.
        '</span></span>';

	$currentPage->title=t('Edit: ').$title;


	// ********* date/time

	// For browsers that support <input type="date"/>
	$form['date']=['type'=>'date',
				   'title' => t("Start Time"),
				   'attributes'=>['size'=>13],
				   'default-value'=>($event->startTime ? date('Y-m-d',$event->startTime) : ''),
				  ];

	// For old browsers that do not support <input type="date"/>
	list($dateFmt,$unused)=demosphere_date_time_short_day_month_year();
	$form['date-deprecated']=['type'=>'textfield',
				   'title' => t("Start Time"),
				   'attributes'=>['size'=>13],
				   'default-value'=>
				   ($event->startTime ? 
					demos_format_date('short-day-month-year',$event->startTime) : 
					strftime(str_replace('%d','?',$dateFmt), time())),
				  ];

	$time=strftime('%R',$event->startTime);
	$form['time']=['type'=>'textfield',
				   'attributes'=>['size'=>6],
				   'default-value'=>$event->startTime && $time!=='03:33' ? $time : '',
				   'attributes'=>['placeholder'=>t('hh:mm')],
				   'field-suffix'=>
				   '<div id="parsed-dates-times-selectors">'.
				   ($event->body!='' ? 
					demosphere_event_parsed_dates_selector($event->body).
					demosphere_event_parsed_times_selector($event->body) : '').
				   '</div>'.
				   '<button type="button" id="date-suggest">'.t('suggest').'</button>',
				  ];
	
	if($user->checkRoles('admin','moderator'))
	{
		$form['events-near-date']=['html'=>'<div id="eventsNearDate"></div>',];
	}

	// ********* text matching

	if($user->checkRoles('admin','moderator'))
	{
		$tm='<div id="text-matching">';
		if($event->id)
		{
			require_once 'text-matching/text-matching.php';
			$doc=TMDocument::getByTid('event',$event->id);

			// Create bogus TMDocument in case there are no real matching docs, only "extra" 
			if($doc===null){$doc=new TMDocument('event',$event->id);}

			// Find extra docs by looking into event edit log for pasted or imported email/article sources
			$extra1=[];
			$logMessages=$event->logText();
			if(preg_match_all('@imported (selection from )?'.
							  'feed article ([0-9]+)@'          ,$logMessages,$m)){foreach($m[2] as $id){$extra1[$id]='feed_import';}}
			if(preg_match_all('@imported (selection from )?'.
							  'email ([0-9]+)@'		            ,$logMessages,$m)){foreach($m[2] as $id){$extra1[$id]='mail_import';}}
			if(preg_match_all('@Paste from article ([0-9]+)@'	,$logMessages,$m)){foreach($m[1] as $id){$extra1[$id]='feed_import';}}
			if(preg_match_all('@Paste from email ([0-9]+)@'		,$logMessages,$m)){foreach($m[1] as $id){$extra1[$id]='mail_import';}}
			if(preg_match_all('@received-ref-([0-9]+)@'		    ,$logMessages,$m)){foreach($m[1] as $id){$extra1[$id]='mail_import';}}
			$extra=[];
			foreach($extra1 as $tid=>$type)
			{
				$id=db_result_check("SELECT id FROM TMDocument WHERE tid=%d AND type='%s'",$tid,$type);
				if($id!==false){$extra[]=$id;}
			}
			$tm.=$doc->displaySimilarDocInfo(false,
											 $event->startTime>0 ? 
											 [$event->startTime] : 
											 [],
											 $extra,true);
		}
		$tm.='</div>';
		$form['text-matching'] = ['html'=>$tm];
	}

	// ********* place
	
	$form['place-and-public-form-info-start']=['html'=>'<div id="place-and-public-form-info">'];
	require_once 'demosphere-place-form.php';
	list($placeForm,$placeFormOpts)=demosphere_place_form_event($event->usePlace(),$event->id,$form['self-edit-ok']['value']);
	$form=array_merge($form,$placeForm);
	$formOpts['pre-render']=$placeFormOpts['pre-render'];

	// ********* event publish form ( ajax edit public form )

	if(isset($event->extraData['public-form']) && $user->checkRoles('admin','moderator'))
	{
		require_once 'demosphere-event-publish-form.php';
		$publicFormData=$event->extraData['public-form'];
		$formOut='';
		$formOut.='<h2>'.t('Public form').'</h2>';
		$formOut.='<div id="public-form-info-contents">';
		$unused=[t('contact'),t('source'),t('remarks'),t('price')];// translation
		$found=[];
		if(val($publicFormData,'is-politis'))
		{
			$formOut.=t('This event was submitted from Politis');
		}
		foreach(['contact','source','price','remarks'] as $f)
		{
			if(!isset($publicFormData[$f])){continue;}
			if(!strlen($publicFormData[$f])){continue;}
			$formOut.='<div>';
			$formOut.='<h3>'.t($f).': </h3><p>';
			if($f==='price'){$prices=demosphere_event_publish_form_prices();$formOut.=ent($prices[$publicFormData[$f]]);}
			else
			{
				require_once 'dlib/filter-xss.php';
				$v=str_replace("\n",'<br/>',ent($publicFormData[$f]));

				$httpUrl='https?://[^[:space:]<>"\']*[^[:space:]<>"\'.]';
				$v=preg_replace_callback('@\b'.$httpUrl.'@i',function($m)
				{
					$url=$m[0];
					$url=demosphere_url_remove_tracking($url);
					$shortUrl=$url;
					$shortUrl=urldecode($shortUrl);
					$shortUrl=dlib_cleanup_plain_text($shortUrl,false,false);// urldecode can make invalid utf8
					$shortUrl=dlib_truncate($shortUrl,45);
					return '<a href="'.ent($url).'">'.ent($shortUrl).'</a>';
				},$v);	

				$email='[A-Z0-9._%+-]+\@[A-Z0-9.-]+\.[A-Z]{2,10}';
				$v=preg_replace_callback('@\b'.$email.'\b@i',function($m)
				{
					global $base_url;
					return '<a target="_blank" '.
						'href="'.$base_url.'/search?search=%22'.urlencode(html_entity_decode($m[0])).'%22&amp;event-date=all">'.ent($m[0]).'</a>';
				},$v);
				$formOut.= filter_xss($v);
			}
			$formOut.='</p></div>';
		}
		if($formOut==''){$formOut='<p>'.t('No information was provided.').'</p>';}
		$formOut.='<p id="tech-data">';
		$formOut.=demos_format_date('shorter-date-time',$event->created);
		if($user->checkRoles('admin') && isset($publicFormData['tempId']))
		{
			$formOut.=' - '.ent(val($publicFormData,'tempId',''));
			if(isset($publicFormData['ip']))
			{
				$ipSearchUrl=$base_url.'/search?search='.urlencode('"'.$publicFormData['ip'].'"').'&event-date=all';
				$formOut.=' / <a href="'.ent($ipSearchUrl).'" target="blank">'.ent($publicFormData['ip']).'</a>';
			}
		}
		$formOut.='</p>';
		$formOut.='</div>';
		$form['public-form-info']=['html' =>'<div id="public-form-info">'.$formOut.'</div>',];
	}


	$form['place-and-public-form-info-end']=['html'=>'</div>'];

	// ********* log

	if($user->checkRoles('admin','moderator'))
	{
		$logAdd='';

		// special case: add public form contact (email) to log if it isn't there yet
		if(isset(                   $event->extraData['public-form'])  && 
		   isset(                   $event->extraData['public-form']['contact']) &&
		   strpos(                  $event->extraData['public-form']['contact'],'@')!==false &&
		   strpos($event->logText(),$event->extraData['public-form']['contact'])===false   )
		{
			$logAdd.="\n"."Public form contact:".$event->extraData['public-form']['contact'];
		}

		$form['log-add']=[
			                 'type'        =>'textarea',
							 'title'       =>t("log of all actions"),
							 'attributes'  =>['cols'=>100,'rows' => 2],
							 'prefix'      =>'<div id="log-button-wrapper"><div id="log-button">'.t('log').'</div></div>',
							 'field-prefix'=>'<div id="log-display">'.$event->logRender().'</div>',
							 'default-value'=>$logAdd,
						 ];
	}


	require_once 'htmledit/demosphere-htmledit.php';

	$form['body']=$objForm['body'];
	$form['body']['title' ]=t('event body');
	$form['body']['suffix']=demosphere_htmledit_js_config('edit-body');
	$form['body']['required']=true;
	require_once 'dlib/filter-xss.php';
	$form['body']['default-value']=filter_xss_admin($form['body']['default-value']);

	// ********* topics

	$defaultTopics=array_keys($event->getTopics());
	if(count($defaultTopics)===0 && $demosphere_config['topics_required']!=='mandatory' && ($event->extraData['topic-was-set'] ?? false)){$defaultTopics=[0];}
	$form['topics']=
		['type' => 'select',
		 'title' => t('Topics'),
		 'multiple'=>true,
		 'options' => Topic::getAllNames()+($demosphere_config['topics_required']!=='mandatory' ? [0=>t('others')] : []),
		 'default-value'=>$defaultTopics,
		];

	// ********* terms
	foreach($demosphere_config['term_vocabs'] as $vid=>$vocab)
	{
		$form['vocab-'.$vid]=
			['type' => 'textfield',
			 'title' => $vocab['name'],
			 'attributes'=>['class'=>['vocab','vocab-'.$vocab['type']]],
			 'wrapper-attributes'=>['class'=>['vocab-wrapper']],
			 'description'=>t('Enter a comma separated list of terms. A suggestion will be shown for the last one.'),
			 'default-value'=>implode(', ',$event->getTermNames($vid)),
			];
	}

	// ********* event emails (send an editable, pre-defined email to event organizer)

	if($user->checkRoles('admin','moderator'))
	{
		$form['event_emails']=
			[
				'type' => 'fieldset',
				'title' => t('send an email concerning this event'),
				'collapsed' => 0,
			];

		$form['event_email_address'] =
			['title' => t("Email address(es) that will receive a message"), 
			 'attributessize' => ['size'=>40],
			 'type' => 'textfield',
			 'default-value' => "",
			 'field-suffix'=>'<div id="email-suggestion" style="display:none">'.
			 '<button type="button" id="email-suggest">'.t('suggest').'</button>'.
			 '<ul><li></li></ul>'.
			 '</div>',
			];


		$emails=$demosphere_config['emails'];


		$form['emails-and-history'] = ['html'=>'<div id="emails-and-history">'];
		$form['eventEmails'] = ['html'=>'<ul id="eventEmails">'];
		foreach($emails as $nb=>$email)
		{
			$form['event_email_'.$nb.'_begin'] = 
				['html' => '<li class="eventEmail"><div class="eventEmailHead">'];

			$form['event_email_select_'.$nb] = 
				['type' => 'checkbox',
				 'title' => '',
				 'attributes'=>['class'=>['eventEmailSelect']],
				 'default-value' => $nb==0,
				];

			$form['event_email_'.$nb.'_title'] = 
				['html'=>'<a class="eventEmailTitle" href="javaScript:void(0);">'.
				 ent($email['name']).'</a>'];

			$form['event_email_'.$nb.'_endhead'] = 
				['html'=>'</div><div class="eventEmailDetails">'];

			$form['event_email_subject_'.$nb] =
				['type' => 'textfield',
				 'title' => t("subject of message !number",
							  ['!number'=>$nb]),
				 'attributes'=>['size' => 80],
				 'default-value' => 
				 demosphere_event_edit_form_email_string_replace($email['subject'],$event,false),
				];
			$form['event_email_body_'.$nb] =
				['type' => 'textarea',
				 'title' => t("body of message !number",
							  ['!number'=>$nb]),
				 'attributes'=>['cols' => 80,'rows'=>8],
				 'default-value' => 
				 demosphere_event_edit_form_email_string_replace(trim($email['body']),$event,false),
				];
			$form['event_email_'.$nb.'_endDetails'] = 
				['html'=>'</div>'];
			$form['event_email_'.$nb.'_end'] = ['html' => '</li>'];

		}

		$form['eventEmailsEnd'] = ['html'=>'</ul>'];

		$form['event_email_history'] = ['html'=>demosphere_event_edit_form_email_history($event)];
		$form['emails-and-history-end'] = ['html'=>'</div>'];

		$selfEditListOut='';
		if($event->id)
		{
			$selfEditList=demosphere_self_edit_get_list($event->id,$event->repetitionGroupId);
			foreach($selfEditList as $selfEdit)
			{
				$selfEditListOut.='<span id="self-edit-'.$selfEdit['password'].'" class="self-edit" title="'.
					'created:'.  ent(demos_format_date('short-date-time',$selfEdit['created'])).'" '.
					' data-password="'.ent($selfEdit['password']).'" '.
					' data-event-id="'.intval($selfEdit['event']).'" '.
					' data-group-id="'.intval($selfEdit['repetition_group']).'" '.
					'><span class="self-edit-delete"></span>'.ent($selfEdit['email']).'</span>, ';
			}
			$selfEditListOut=preg_replace('@\s*,\s*$@','',$selfEditListOut);
		}
		$selfEditListOut.='<div id="self-edit-popup">'.
			'<h4 id="sepu-title">'.t('self edit allowed for:').'<br/><span id="sepu-email"></span></h4>'.
			'<div id="sepu-event"><input type="checkbox"> '.t('edit this event').'</div>'.
			'<div id="sepu-group"><input type="checkbox"> '.t('edit repetition group').'</div>'.
			'</div>';

		$form['self-edit'] = 
			['html'=>$selfEditListOut==='' ? '':
			 '<div id="self-edit-list">'.t('self edit allowed for:').' '.$selfEditListOut.'</div>'
			 ,];

		$form['event_emails-end' ]=['type' => 'fieldset-end'];

	}

	// ********* moderation status / message + needs attention

	$form['moderation']=
		[
			'type' => 'fieldset',
			'title' => t('publishing status'),
			'collapsed' => 0,
		];

	if($user->checkRoles('admin','moderator'))
	{
		$modMessageSelect='<select id="moderation-message-select">';
		$modMessageSelect.='<option value="">---</option>';
		foreach($demosphere_config['moderation_messages'] as $msg)
		{
			$msg=str_replace('%date',demos_format_date('short-day-month',time()),$msg);
			$modMessageSelect.='<option>'.ent($msg).'</option>';
		}
		$modMessageSelect.='</select>';
		$form['moderationMessage']=$objForm['moderationMessage'];
		$form['moderationMessage']['type']='textarea';
		$form['moderationMessage']['title'] = t('Moderation message');
		$form['moderationMessage']['attributes']=['cols' => 30,'rows'=>4];
		$form['moderationMessage']['field-suffix']=$modMessageSelect;

		$form['needsAttention']=$objForm['needsAttention'];
		$form['needsAttention']['title' ]= t('Needs attention');
		$form['needsAttention']['options' ]= Event::needsAttentionTitles();
		$form['needsAttention']['attributes']=['class'=>['needs-attention']];
	}

	$mstatusVal=$event->moderationStatus;
	if($mstatusVal==0){$mstatusVal=1;}// created->waiting

	// When creating new event, set status to published 
	if(!$event->id){$mstatusVal=Event::moderationStatusEnum('published');}

	$optstatus=Event::moderationStatusLabels();
	unset($optstatus[Event::moderationStatusEnum('created')]);
	if(!$user->checkRoles('admin','moderator')   ){unset($optstatus[Event::moderationStatusEnum('waiting')]);}
	if(!$user->checkRoles('admin','moderator') ||
	   !$demosphere_config['use_rejected_shown']) {unset($optstatus[Event::moderationStatusEnum('rejected-shown')]);}
	if(!$user->checkRoles('admin','moderator')   ){unset($optstatus[Event::moderationStatusEnum('rejected')]);}
	unset($optstatus[Event::moderationStatusEnum('trash')]);

	// Special case: edit un-published event by user with no auth (self-edit): cannot change moderationStatus
	if($user->checkRoles('none') && $event->id &&  $event->status===0)
	{
		// self-edit unpublished
		$optstatus=[];
	}

	if(count($optstatus)>0)
	{
		$form['moderationStatus']=$objForm['moderationStatus'];
		$form['moderationStatus']['type' ]= 'radios';
		$form['moderationStatus']['title' ]= t('Moderation status');
		$form['moderationStatus']['default-value' ]= $mstatusVal;
		$form['moderationStatus']['options' ]= dlib_array_column($optstatus,'title');
		$form['moderationStatus']['attributes']=['class'=>['moderation-status']];
	}
	else
	{
		$form['moderationStatus']['type']='hidden';
		$form['moderationStatus']['value']=$mstatusVal;
	}

	if(isset($optstatus[Event::moderationStatusEnum('incomplete')]))
	{
		$form['incompleteMessage']=$objForm['incompleteMessage'];
		$form['incompleteMessage']['type' ]= 'textarea';
		unset($form['incompleteMessage']['title']);
		$incompleteSelect='<select id="icomplete-message-select">';
		$incompleteSelect.='<option>-- '.t('Why is it incomplete ?').' -- </option>';
		foreach($demosphere_config['incomplete_messages'] as $msg){$incompleteSelect.='<option>'.ent($msg['name']).'</option>';}
		$incompleteSelect.='<option>'.t('custom').'</option>';
		$incompleteSelect.='</select>';
		$currentPage->addJsVar('incomplete_messages',$demosphere_config['incomplete_messages']);
		$form['incompleteMessage']['field-prefix']=$incompleteSelect;
		// HTML block for inline tinymce. This is filled in JS at page startup.
		$form['incompleteMessage']['field-suffix']='<div id="incomplete-message-html"></div>';
	}

	$form['moderation-end']=['type' => 'fieldset-end',
							 'after'=>'moderationStatus'];

	// Remove fieldset if no mod status form items available
	if( isset($form['moderationStatus']['value']) && 
	   !isset($form['needsAttention'])     )
	{
		unset($form['moderation']);
		unset($form['moderation-end']);
	}

	// ********** extra-options **************

	$form['extra-options'] = [
		'type' => 'fieldset',
		'title' => t('extra options'),
		'collapsible' => true,
		'collapsed' => true,
		'attributes'=>['class'=>['extra-options']],
							 ];


	// ************** send-urgent-event

	if($user->checkRoles('admin','moderator') && 
	   ($event->startTime==0 || $event->startTime<time()+48*3600))
	{ 
		$form['send-urgent-event']=
			[
				'type' => 'checkbox',
				'title' => t('Send urgent event email'),
				'description' => t('Send an email with this event to users that have subscribed to emails. This is only possible for events that are less than 48 hours away. Please only use this system exceptionally and respectfully. Avoid spamming your users with large number of emails. Abuse might get the email server blacklisted.'),
			];
	}

	// ************** showOnFrontpage

	$form['showOnFrontpage']=$objForm['showOnFrontpage'];
	$form['showOnFrontpage']['title']=t('Show on frontpage');

	// special case: click from widget config adds GET option dontShowOnFrontpage, so that
	// showOnFrontpage defaults to false
	if(isset($_GET['dontShowOnFrontpage']))
	{
		$form['showOnFrontpage']['default-value']=false;
	}

	// non frontpage publishers create private events by default 
	if($event->id==0 && !$user->checkRoles('admin','moderator','frontpage event creator'))
	{
		$form['showOnFrontpage']['default-value']=false;
	}

	// Non frontpage publishers cannot change showOnFrontpage
	// (but their event might have been shown on frontpage by an admin).
	if(!$user->checkRoles('admin','moderator','frontpage event creator'))
	{
		$form['showOnFrontpage']['type' ]='hidden';
		$form['showOnFrontpage']['value']=$form['showOnFrontpage']['default-value'];
	}

	// move "showOnFrontpage" into main form for private events
	if($form['showOnFrontpage']['default-value']!=true &&
	   $user->checkRoles('admin','moderator','frontpage event creator'))
	{
		$item=$form['showOnFrontpage'];
		unset($form['showOnFrontpage']);
		$form=dlib_array_insert_assoc($form,'moderationStatus','showOnFrontpage',$item,true);
	}	

	if($user->checkRoles('admin','moderator'))
	{
		$form['author-fieldset'] = [
			'type' => 'fieldset',
			'title' => t('Author & last editor'),
			'collapsible' => true,
			'collapsed' => true,
								   ];
		$form['authorId']=$objForm['authorId'];
		$form['authorId']['type']='textfield';
		$form['authorId']['title']=t('Author / creator');
		$form['authorId']['default-value']=$event->authorId==0 ? '' : User::fetch($event->authorId)->login;
		$form['authorId']['validate']=function($v){return $v!='' && User::fetchByLogin($v,false)===null ? t('Invalid user name'):true;};
		$form['authorId']['pre-submit']=function(&$v){$v=$v=='' ? 0 : User::fetchByLogin($v)->id;};

		$form['created_ro']=$objForm['created'];
		$form['created_ro']['attributes']['readonly']=true;
		$form['changed_by_ro']=['type'=>'textfield',
								'title'=>t('Changed by (last editor)'),
								'value'=>$event->changedById==0 ? '' : User::fetch($event->changedById)->login,
								'attributes'=>['readonly'=>true]];
		$form['changed_ro']=$objForm['changed'];
		$form['changed_ro']['attributes']['readonly']=true;
		$form['author-fieldset-end'] = ['type' => 'fieldset-end',];
	}

	// Seo fields are only for admin and moderator
	if($user->checkRoles('admin','moderator'))
	{
		$form['seo-fieldset'] = [
			'type' => 'fieldset',
			'title' => t('SEO - title, keywords, description'),
			'collapsible' => true,
			'collapsed' => true,
								];
		$form['seo-title'] = 
			[
				'type' => 'textfield', 
				'title' => 'custom title',
				'default-value' => val($event->extraData,'seo-title'),
				'description' => t('If not empty, this will replace the title tag which shows up in search-engine results and on browser windows. This can be important for SEO. Google search results only show the first 65 characters. Other search engines might show more. This might change.'),
			];

		$form['seo-keywords'] = 
			[
				'type' => 'textfield', 
				'title' => 'custom keywords',
				'default-value' => val($event->extraData,'seo-keywords'),
				'description' => t('If not empty, this will replace the meta keywords. You should use a comma separated list of keywords.'),
			];

		$form['seo-description'] = 
			[
				'type' => 'textfield', 
				'title' => 'custom description',
				'attributes'=>['maxlength'=>200],
				'default-value' => val($event->extraData,'seo-description'),
				'description' => t('If not empty, this will replace the meta description. Google shows the first 156 characters in search results.'),
			];
		$form['seo-fieldset-end' ]=['type' => 'fieldset-end'];
	}

	if(count($demosphere_config['post_on_other_site']) &&
	   $user->checkRoles('admin','moderator'))
	{
		$form['post_on_other_site']=['type' => 'fieldset',
									 'title' => t('Post on other site'),
									 'collapsible' => true,
									 'collapsed' => false,
									];
									  
		foreach($demosphere_config['post_on_other_site'] as $n=>$site)
		{
			$form['post_on_other_site_'.intval($n)] = 
				['type' => 'checkbox',
				 'title' => $site['url'],
				];
		}
		$form['post_on_other_site_end']=['type' => 'fieldset-end',];
	}
	if($user->checkRoles('admin','moderator'))
	{
		$form['revision']=['type'=>'checkbox',
						   'title'=>t('Create revision')
						  ];
	}

	if($user->checkRoles('admin','moderator'))
	{
		$form['duplicate']=['type'=>'textfield',
							'title'=>t('Duplicate'),
							'default-value'=>val($event->extraData,'duplicate'),
							'description'=>t("Non moderators will be redirected to the event you specify here. This event must be un-published. You can enter a link to the event or the number of the event. Example: @example",['@example'=>$base_url.'/'.$demosphere_config['event_url_name'].'/1234'])
						   ];
	}

	if($user->checkRoles('admin','moderator'))
	{
		$form['useCarpool']=$objForm['useCarpool'];
		$form['useCarpool']['title']=t('Enable share-a-ride for this event');
	}

	if($user->checkRoles('admin','moderator'))
	{
		$form['smaller-images']=['type'=>'checkbox',
								 'title'=>t('Make images lighter-weight'),];
	}

	// only used to check if event changed by other user during save
	$form['changed-check']=['type'=>'hidden',
							'default-value'=>$event->changed];

	$form['extra-options-end']=['type' => 'fieldset-end'];

	// ***  remove fieldset if it is empty
	$fieldsetIsEmpty=true;
	foreach(array_keys($form) as $p=>$k)
	{
		if($p>dlib_key_pos($form,'extra-options') && 
		   $p<dlib_key_pos($form,'extra-options-end') &&
		   val($form[$k],'type')!=='hidden'){$fieldsetIsEmpty=false;}
	}
	if($fieldsetIsEmpty)
	{
		unset($form['extra-options']);
		unset($form['extra-options-end']);
	}

	$form['js-event-id'] = ['type'=>'hidden','value'=>$event->id,];
	$form['js-repetition-group-id'] = ['type'=>'hidden','value'=>$event->repetitionGroupId,];

	// Hack to create an onsubmit handler that fires before wyswyig/tinymce.
	// This is used to check for large images (src="data:...") that crash wyswyig/tinymce.
	$form['onsubmit-jshack']=['html'=>'<script type="text/javascript">document.getElementById("Event").setAttribute("onsubmit","return demosphere_event_edit_form.presubmit(event,this);")</script>'];

	$form['save']  =$objForm['save'];
	// disable DBObject form auto save, we do this manually in our submit handler
	unset($form['save']['submit-2']);

	if($user->checkRoles('admin','moderator'))
	{
		$form['trash']=['type' => 'submit', 
						'value' => t('Move to trash'),
						'attributes'=> $event->getModerationStatus()==='trash' ? ['disabled'=>true] : [],
					   ];
	}

	// 10/2015 : Removed delete from normal event edit. Only appears if event is already trashed.
	if($event->getModerationStatus()==='trash' && $event->access('delete') && isset($objForm['delete']))
	{
	 	$form['delete']=$objForm['delete'];
	 	$form['delete']['submit-2']=function()use($event){dlib_redirect($event->url().'/delete');};
	}

	$formOpts['validate']='demosphere_event_edit_form_validate';
	$formOpts['submit']='demosphere_event_edit_form_submit';

	// hack to avoid validation errors for empty fields when trashing, 
	// but still save any changed fields
	if(isset($_POST['trash']))
	{
		if($_POST['title']===''){$_POST['title']='empty';}
		if($_POST['body' ]===''){$_POST['body' ]='empty';}
	}

	return form_process($form,$formOpts);
}

function demosphere_event_edit_form_css_js()
{
	global $demosphere_config,$currentPage,$user;

	$status=Event::getSchema();
	$status=$status['moderationStatus']['values'];

	$currentPage->addJs('lib/jquery.js');
	$currentPage->addJs('lib/jquery-browser.js');
	$currentPage->addJsVar('demosphere_mstatus',$status);
	$currentPage->addJsVar('demosphere_mstatusIdx',array_flip($status));

	// add this here so it comes before demosphere-event-edit-form.js
	$currentPage->addJs( 'dlib/form.js');

	$currentPage->addJs( 'dlib/merge-calls.js');
	$currentPage->addJs( 'lib/jquery-SafeEnter.js');
	$currentPage->addJs( 'demosphere/js/demosphere-misc.js');
	$currentPage->addJs( 'demosphere/js/demosphere-event-edit-form.js');
	require_once 'docconvert/docconvert.php';
	docconvert_js_config();
	$currentPage->addCssTpl('demosphere/css/demosphere-event-edit-form.tpl.css');
	$currentPage->addCssTpl('demosphere/docconvert/docconvert.tpl.css');
	$currentPage->addCssTpl('demosphere/css/demosphere-event-edit-form.tpl.css');
	$currentPage->addCssTpl('demosphere/text-matching/text-matching-info.tpl.css');
	$currentPage->addCssTpl('demosphere/css/demosphere-highlight.tpl.css');
	$currentPage->addJsConfig('demosphere',['event_title_max_width']);

	// Javascript text message translations
	$currentPage->addJsTranslations(
		['text-submit-message'=>t('submitting ... please wait'),
		 'text-ie6-too-old'   =>t('Your browser version (internet explorer 6) is very old. This page will not work correctly. Please use a more recent browser, like firefox.'),
		 'self-edit-delete'=>t('Are you sure you want to remove this users permissions to edit this event?'),
		 'invalid-date-or-time'=>t('invalid date or time'),
		 'title-bold-needs-select'=>t('You need to select the important word in the title, before using this button.'),
		]);
}


/**
 */
function demosphere_event_edit_form_validate(&$form,$activeButton)
{
	global $demosphere_config,$base_url,$user;
	require_once 'demosphere-date-time.php';

	$event=$form['form_object']['data'];
	$values=dlib_array_column($form,'value');

	$isPublished=
		$values['moderationStatus']==Event::moderationStatusEnum('incomplete')	   ||
		$values['moderationStatus']==Event::moderationStatusEnum('published')	   ||
		$values['moderationStatus']==Event::moderationStatusEnum('rejected-shown');

	if($activeButton==='trash'){$isPublished=false;}

	// No long titles in published events
	require_once 'demosphere-event.php';
	$city=$values['city'];
	if(trim($values['city-short'])!==''){$city=$values['city-short'];}
	$titleLengthIsOk=demosphere_event_title_length_is_ok($values['title'],$city);
	if(!$titleLengthIsOk && $isPublished)
	{
		$form['title']['error']=t('the title is too long');
	}

	// only allow no keywords in titles of un-published events
	$nbUnderscores=substr_count($values['title'],'_');
	if($demosphere_config['require_keyword_in_event_title'] && $isPublished &&
	   !($nbUnderscores>=2 && ($nbUnderscores%2)==0))
	{
		$form['title']['error']=t('In the title, you must choose an important word that will be displayed in a different color on the frontpage. To select a word in the title, put an undescore before and after the word. The undescore is located on the top right part of your keyboard (above the minus sign). Example : this is a title with _a keyword_ that is important' );
	}

	// Warn for long words between underscores
	if(preg_match_all("@_([^_]*)_@",$values['title'],$m))
	{
		$l=mb_strlen(implode(' ',$m[1]));
		if($l>13){dlib_message_add(t('In the title, the highlighted words (between underscores "_") are too long. Please try to choose a single, significant, word. This helps your user quickly scan the frontpage for events of interest.'),'warning');}
	}

	// ************** start date & time ***********
	// check if the user specified a valid date and time
	$dateInputSupported=$values['date-deprecated']==='deprecated';

	if($dateInputSupported  && $activeButton!=='trash' && $values['date']===''){$form['date']['error']=ent(t('This field is required'));}

	if($activeButton!=='trash' && !$dateInputSupported &&
	   demosphere_date_time_parse_short($values['date-deprecated'])===false)
	{
		$form['date-deprecated']['error']=t('Date should be in the format mm/dd/yyyy');
	}
	$time=demosphere_date_time_parse_time($values['time']);
	if($activeButton!=='trash' && $values['time']!=='' &&
	   $time===false)
	{
		$form['time']['error']=t('Time should be in the format hh:mm');
	}

	if($time===false){$time=['hours'=>3,'minutes'=>33];}
	if($dateInputSupported){$startTime=strtotime($values['date'].'T'.sprintf('%02d',$time['hours']).':'.sprintf('%02d',$time['minutes']));}
	else                   {$startTime=(int)demosphere_date_time_parse_short($values['date-deprecated'],$values['time']);}

	// Only admin and mods can change date of past event
	if(!$user->checkRoles('admin','moderator') &&
	   $event->startTime!=0 && // new event created by fp or priv ev user
	   strtotime('3:33',$event->startTime) < strtotime('3:33') &&
	   $startTime!==0 && 
	   date('d/m/Y',$startTime)!==date('d/m/Y',$event->startTime))
	{
		$form['date'.($dateInputSupported ? '' : '-deprecated')]['error']=t('You cannot change the date of a past, archived event. Please create a new event.');
	}

	demosphere_place_form_event_validate($form);

	// only allow unset city in un-published events
	if(trim($values['city'])==='' && $isPublished)
	{
		$form['city']['error']=t('To publish, you must fill the city field' );
	}

	// only allow empty addresses in un-published events
	if(trim($values['address'])=='' && $isPublished)
	{
		$form['address']['error']=t('to publish, you must fill the address field');
	}

	// only allow unset topic in un-published events
	if(count($values['topics'])==0  && $isPublished && $demosphere_config['topics_required']!=='not-required')
	{
		$form['topics']['error']=t('To publish, you must select a topic.' );
	}

	// ************** validate email fields ***********
	if(isset($form['event_emails'])){demosphere_event_edit_form_send_all_emails($form,false,['isPublished'=>$isPublished]);}

	// check if event changed by other user during save
	if($event->id && $event->changed>$values['changed-check'])
	{
		$form['changed-check']['error']=t('This event has been modified by another user. Your changes cannot be saved.' );
	} 

	// ************** urgent email ***********
	if(val($values,'send-urgent-event') && !$isPublished)
	{
		$form['send-urgent-event']['error']=t('Urgent event email can only be sent for published events.');
	}
	if(val($values,'send-urgent-event') && ($startTime>time()+48*3600 || $startTime < time()-3600*24))
	{
		$form['send-urgent-event']['error']=t('Urgent event email can only be sent for events that are less than 48 hours away.');
	}

	require_once 'dlib/mail-and-mime.php';
	if(val($values,'send-urgent-event') && dlib_throttle_count('send-urgent-event',3600*7)>1)
	{
		$form['send-urgent-event']['error']=t('Too many urgent event emails sent. Please avoid spamming your users.');
	}

	// ************** duplicate ***********
	if(isset($values['duplicate']) && trim($values['duplicate'])!=='')
	{
		if(!preg_match('@^ *('.preg_quote($base_url,'@').'/)?(/?'.$demosphere_config['event_url_name'].'/)?([0-9]+)(\s*$|[?/])@',
					   $values['duplicate'],$matches)  || $matches[3]==$event->id)
		{
			$form['duplicate']['error']=t('invalid link or event number');
		}
		else
		{
			$dup=Event::fetch($matches[3],false);
			if($dup===null){$form['duplicate']['error']=t('duplicate event does not exist');}
		}
	}
}

//! This hook is configured in form $formOpts
function demosphere_event_edit_form_submit($form,$formOpts,$activeButton)
{
	global $base_url,$demosphere_config,$user,$custom_config;
	require_once 'demosphere-date-time.php';

	$values=dlib_array_column($form,'value');

	$event=$form['form_object']['data'];

	// **** Keep a copy of old event for change detection
	$oldEvent    =clone $event;
	$oldEvent->id=$event->id;
	$oldEvent->setPlace(clone $event->usePlace());
	// hack : old topics need to be stored 
	$oldEvent->extraData['topics']=$event->getTopicNames();
	$oldEvent->extraData['terms' ]=$event->getAllTerms();

	// Check if this is a Selfedit
	$selfEditOk=$values['self-edit-ok'];

	// Add a revision for existing events that are changed by a different user (or self-edited)
	// Note: revisions create needs to be done before place save.
	if($event->id!=0 && 
	   $event->changedById==$user->id && // avoid duplicate revision creation if it will be created during $event->save()
	   ($selfEditOk ||
		val($values,'revision')))
	{
		require_once 'demosphere-event-revisions.php';
		demosphere_event_revisions_create($event->id);
	}

	// **** Copy form post values to fields (also casts to correct types)
	$event->dboCopyFromStringArray($values);
	$event->startTime=false;

	if($activeButton=='trash'){$event->setModerationStatus('trash');}

	// **** set event status from event moderationStatus
	$event->status=
		$values['moderationStatus']==Event::moderationStatusEnum('incomplete'	  ) ||
		$values['moderationStatus']==Event::moderationStatusEnum('published'	  ) ||
		$values['moderationStatus']==Event::moderationStatusEnum('rejected-shown') 
		? 1:0;

	if($event->status && isset($values['duplicate']) && trim($values['duplicate'])!=='' )
	{
		dlib_message_add(t('Unpublishing duplicate event.'),'warning');
		$values['moderationStatus']==Event::moderationStatusEnum('rejected'); 
		$event->setModerationStatus('rejected');
		$event->status=0;
	}
	// **** set showOnFrontpage for reject shown
	if($event->getModerationStatus()=='rejected-shown')
	{
		$event->showOnFrontpage=false;
	}

	// **** set needs attention
	if(!$user->checkRoles('admin','moderator')){$event->setNeedsAttention('non-event-admin');}
	if($selfEditOk                            ){$event->setNeedsAttention('self-edited'    );}
	// very special case for opinions: self edit of an unpublished publication request should keep needs-attention=publication-request
	if($selfEditOk && !$event->status && $oldEvent->id && 
	   ($oldEvent->getNeedsAttention()==='publication-request' ||
		$oldEvent->getNeedsAttention()==='publication-request-waiting')){$event->needsAttention=$oldEvent->needsAttention;}

	// **** cleanup title and compute htmlTitle
	$event->title=dlib_cleanup_plain_text($event->title,false);
	$event->title=preg_replace('@\s+@',' ',$event->title);
	$htmlTitle=htmlspecialchars($event->title);
	$htmlTitle=preg_replace('@_([^_]*)_@','<strong>$1</strong>',
							 $htmlTitle);
	$event->title=str_replace('_','',$event->title);
	$event->htmlTitle=$htmlTitle;

	// **** parse start time
	$dateInputSupported=$values['date-deprecated']==='deprecated';
	$time=demosphere_date_time_parse_time($values['time']);
	if($time===false){$time=['hours'=>3,'minutes'=>33];}

	if($dateInputSupported){$event->startTime=strtotime($values['date'].'T'.sprintf('%02d',$time['hours']).':'.sprintf('%02d',$time['minutes']));}
	else                   {$event->startTime=(int)demosphere_date_time_parse_short($values['date-deprecated'],$values['time']);}


	$place=demosphere_place_form_event_submit($form);
	$event->setPlace($place);

	// ***** process body html

	// Add any manually (or js) entered log
	if(isset($values['log-add']) && trim($values['log-add'])!==''){$event->logAdd(dlib_cleanup_plain_text(trim($values['log-add'])));}

	require_once 'htmledit/demosphere-htmledit.php';
	$event->body=demosphere_htmledit_submit_cleanup($event->body);

	if($values['smaller-images'] ?? false)
	{
		$event->body=demosphere_event_edit_smaller_images($event->body);
	}

	// ***** moderationMessage
	$event->moderationMessage=dlib_cleanup_plain_text($event->moderationMessage,false);

	// ***** incompleteMessage
	$event->incompleteMessage=demosphere_htmledit_submit_cleanup($event->incompleteMessage);

	// ***** custom title tag or meta keywords/description
	if(isset($form['seo-fieldset']))
	{
		foreach(['seo-title','seo-keywords','seo-description'] as $f)
		{
			if($values[$f]!==''){$event->extraData[$f]=dlib_cleanup_plain_text($values[$f],false);}
		}
	}

	if(isset($values['duplicate']))
	{
		if(preg_match('@^ *('.preg_quote($base_url,'@').'/)?(/?'.$demosphere_config['event_url_name'].'/)?([0-9]+)(\s*$|[?/])@',
					  $values['duplicate'],$matches))
		{
			$event->extraData['duplicate']=intval($matches[3]);
		}
		else{unset($event->extraData['duplicate']);}
	}

	// ************************* 
	// ***** save event : after this we have a real $event->id
	// ************************* 
	$event->save();

	if($demosphere_config['topics_required']!=='mandatory'  && count($values['topics'])){$event->extraData['topic-was-set']=true;}
	$values['topics']=array_diff($values['topics'],[0]);
	$event->setTopics($values['topics']);

	// terms
	$terms=[];
	foreach($demosphere_config['term_vocabs'] as $vid=>$vocab)
	{
		$terms=array_merge($terms,Term::parse($values['vocab-'.$vid],$vid));
	}
	$event->setTerms($terms);

	// **** log: determine which parts of this event have changed
	$changedMsg=false;
	if($oldEvent->id)
	{
		$changedMsg='(';
		if($event->moderationStatus!=$oldEvent->moderationStatus)
		{
			$changedMsg.='status:'.$event->getModerationStatus().',';
		}
		if($event->showOnFrontpage!=$oldEvent->showOnFrontpage   ){$changedMsg.="showOnFrontpage,";}
		if($event->title      !=$oldEvent->title	 ){$changedMsg.="title,";}
		if($event->body		  !=$oldEvent->body 	 ){$changedMsg.="body,";}
		if(!$event->usePlace()->equals($oldEvent->usePlace())  ){$changedMsg.="place,";}
		if($event->startTime !=$oldEvent->startTime){$changedMsg.="date-time,";}
		if(count(array_diff($values['topics'],array_keys($oldEvent->extraData['topics']))))
		{$changedMsg.="topics,";}
		if(count(array_diff($terms           ,array_keys($oldEvent->extraData['terms' ]))))
		{$changedMsg.="terms,";}
		$changedMsg.=")";
		// add a message this event's log 
		if($selfEditOk===false)
		{
			$event->logAdd('edited '.$changedMsg);
		}
		else
		{
			$selfEditPass=demosphere_self_edit_check_event($event->id,false,true);
			$selfEditEmail=db_result_check("SELECT email FROM self_edit WHERE password='%s'",$selfEditPass);
			$event->logAdd('Selfedit: '.($selfEditEmail!==false ? '['.$selfEditEmail.']' : '').' '.$changedMsg);
		}
	}
	else
	{
		$event->logAdd("created");
		if(!$user->checkRoles('admin','moderator'))
		{
			$event->logAdd("created by non event admin: ".$user->email);
		}
	}

	// **** needs attention rules 
	
	// auto change to "ok", when "publication request" is "published"
	if($oldEvent->id && $user->checkRoles('admin','moderator') &&
	   ($oldEvent->getNeedsAttention()==='publication-request') &&
	   ($event->getModerationStatus()==='published' ||
		$event->getModerationStatus()==='incomplete'))
	{
		$event->setNeedsAttention('ok');
	}
	// auto change to "publication-request-waiting", when "publication request" is "rejected"
	if($oldEvent!==false && $user->checkRoles('admin','moderator') &&
	   ($oldEvent->getNeedsAttention()==='publication-request' &&
		$oldEvent->getModerationStatus()==='waiting') &&
	   ($event->getModerationStatus()==='trash' ||
		$event->getModerationStatus()==='rejected'))
	{
		$event->setNeedsAttention('publication-request-waiting');
	}

	// auto change to Ok if duplicate
	if(isset($event->extraData['duplicate']))
	{
		$event->needsAttention=0;
	}

	// make sure that editing a trash Event (or moving to trash from form)
	// always call $event->trash() to remove this Event from indexing
	if($event->getModerationStatus()==='trash')
	{
		$event->trash();
	}

	// FIXME: maybe we should check for log edit conflict : 
	// log has changed while user was editing this event.
	// This happens, for example, if new mail related to this event is received
	// while this event is being edited. 
	// (isn't this quite rare, and not very important ?)

	// ****  emails are sent from here as they require a valid nid for links
	if(isset($form['event_emails'])){demosphere_event_edit_form_send_all_emails($form,$event);}

	if(val($values,'send-urgent-event'))
	{
		require_once 'demosphere-email-subscription.php';
		demosphere_email_subscription_send_urgent_event($event);
		require_once 'dlib/mail-and-mime.php';
		dlib_throttle_add('send-urgent-event');
	}

	// **** When a non-admin adds or edits an event, send an email to admins so they know about it.
	if(!$user->checkRoles('admin','moderator'))
	{
		// add edit link in message for self edit:
		// now send an email to tell moderators an event has been submitted
		$type=$selfEditOk ? 'self-edit' : 'non-admin-event';
		$subject=mb_substr(($event->title),0,100);
		
		$msg='';
		$msg.='<h3>'.t('This event was edited by a user').'</h3>';
		$msg.='<p>'.t('You can click on this link to check if it is ok.').'<br/>';
		$url=$event->url();
		$msg.='<a href="'.ent($url).'">'.ent($url).'</a>';
		$msg.="<p>Changed: ".$changedMsg."</p>";
		$msg.="<p>".ent(t('Start time:').' '.demos_format_date('full-date-time',$event->startTime))."</p>";
		$msg.="<hr/>";
		$msg.='<h2>'.ent($event->title).'</h2>';
		$msg.=$event->body;

		require_once 'demosphere-emails.php';
		demosphere_emails_notification_send($type,$subject,$msg);
	}

	// ****  when a non admin user creates an event, auto-select it in his user cal
	if($user->id!=0 && !$user->checkRoles('admin','moderator') && !$oldEvent->id)
	{
		require_once 'demosphere-user-calendar.php';
		demosphere_user_calendar_set_select($event->id,$user->id,1);
	}

	// **** post on other site
	if(isset($values['post_on_other_site']))
	{
		foreach($demosphere_config['post_on_other_site'] as $siteNb=>$site)
		{
			if($values['post_on_other_site_'.$siteNb])
			{
				require_once 'demosphere-event-post-on-other-site.php';
				dlib_message_add('posting on site '.$siteNb);
				demosphere_event_post_on_other_site($demosphere_config['post_on_other_site'][$siteNb],$event);
			}
		}
	}

	if(isset($custom_config['custom_demosphere_event_edit_form_submit']))
	{
		$custom_config['custom_demosphere_event_edit_form_submit']($event,$form);
	}

	// special case: opinions of duplicate event
	if(isset($event->extraData['duplicate']) && $demosphere_config['enable_opinions'])
	{
		require_once 'demosphere-opinion.php';
		demosphere_opinions_duplicate($event->id,$event->extraData['duplicate']);
	}


	// **** second save after all steps that needed $event->id
	$event->save();

	// ****
	// special case: a user changed an automatic repetition that is not published. warn him
	if(!$event->status && $selfEditOk && demosphere_self_edit_check_group($event->repetitionGroupId))
	{
		dlib_message_add(t('Warning: You have changed this event but it is NOT published. To publish it, you need to confirm it by going to the repetitions tab and pressing on the "Save" button.'),'warning');
	}

	// ***** 
	// if we save an event belonging to a repetition we redirect user 
	// to a page allowing him to copy changed fields to other events of this repetition group
	if(count($event->getAllRepetitions())>1 && 
	   ($user->checkRoles('admin','moderator') ||
		demosphere_self_edit_check_group($event->repetitionGroupId)))
	{
		// save old event in persistent variable, so that repetition copy form can know what changed
		$oldEvent->extraData['repetition_copy_ts']=time();
		variable_set($event->id.'-'.hash('sha256','erc:'.session_id()),'event-repetition-copy',$oldEvent);
		dlib_redirect($event->url().'/repetition-copy?just-changed');
	}

	// It is safe to redirect here, since this is the last submit handler
	dlib_redirect($event->url());
}

function demosphere_event_edit_smaller_images($html)
{
	$doc=dlib_dom_from_html($html,$xpath);
	foreach($xpath->query("//img/@src") as $attr)
	{
		$src=$attr->value;
		if(!preg_match('@^/files/import-images/(?P<base>(?P<resized>resized[0-9]?[0-9]?-)?(?P<orig>[a-z0-9]+))\.png$@',$src,$m)){continue;}
		$cmd='convert '.escapeshellarg('files/import-images/'.$m['base'].'.png').' '.
			escapeshellarg('files/import-images/'.$m['base'].'.jpg');
		$ret=dlib_logexec($cmd);
		if($ret==0){$attr->value='/files/import-images/'.$m['base'].'.jpg';}
		if($ret!=0){dlib_message_add('smaller-images: failed to convert image:'.ent($cmd));}
		if($m['resized']!=='')
		{
			$cmd='convert '.escapeshellarg('files/import-images/'.$m['orig'].'.png').' '.
				escapeshellarg('files/import-images/'.$m['orig'].'.jpg');
			$ret=dlib_logexec($cmd);
			if($ret!=0){dlib_message_add('smaller-images: failed to convert original image:'.ent($cmd));}
		}
	}
	$html=dlib_dom_to_html($doc);
	return $html;
}


function demosphere_event_edit_form_email_history($event)
{
	global $base_url;
	$emails=[];
	// sent mail 1 (merci (permettre éditer))  to 'xyz@abc.de'
	foreach($event->log as $logEntry)
	{
		preg_match_all('@sent mail (?P<mailNb>[0-9]+) \((?P<mailType>[^\n]*)\) * to \'(?P<to>[^\']+)\'[^\n]*(?P<body>(\n\s*> [^\n]*)*)@s',
					   $logEntry['message'],$matches);
		foreach($matches[0] as $matchNb=>$unused)
		{
			$email=[];
			$email['type'   ]='event-form';
			$email['date'   ]=$logEntry['ts'];
			$email['from'	]=$logEntry['name'];
			$email['to'     ]=$matches['to'      ][$matchNb];
			$email['subject']=$matches['mailType'][$matchNb];
			$email['body'   ]=$matches['body'    ][$matchNb];
			$email['body'   ]=preg_replace('@^\s*> @m','',$email['body']);
			$email['body'   ]=trim($email['body']);
			$emails[]=$email;
		}
	}

	preg_match_all('@(Paste from email |imported (selection from )?email |received-ref-)([0-9]+)@',$event->logText(),$matches);
	foreach(array_unique($matches[3]) as $msgId)
	{
		require_once 'dcomponent/dcomponent-common.php';
		$msg=Message::fetch($msgId,false);
		if($msg===null){continue;}
		$email=[];
		$email['type'   ]='mail-import';
		$email['url'    ]=$msg->url();
		$email['date'   ]=$msg->date;
		$email['from'	]=$msg->from;
		$email['to'     ]=$msg->firstTo;
		$email['subject']=$msg->subject;
		$emailBody=$msg->getFullHtml(false);
		$emailBody=mb_substr($emailBody,0,20000);
		$emailBody=preg_replace('@(\n[ \t]*){4,}@',"\n\n\n",Html2Text::convert($emailBody));
		if(mb_strlen($emailBody)>10000){$emailBody=mb_substr($emailBody,0,10000)."\n...\n";}
		$email['body'   ]=$emailBody;
		$emails[]=$email;
	}
	usort($emails,function($a,$b){return 2*intval($a['date']<$b['date'])-1;});

	if(count($emails)===0){return '';}
	$out='';
	$out.='<div id="email-history">';
	$out.='<h4>'.ent('Past emails').'</h4>';
	$out.='<ul>';
	foreach($emails as $email)
	{
		$out.='<li class="email-history-email type-'.$email['type'].'">';
		$out.='<div class="first-line">';
		$out.='<span class="date"    >'.
			ent(demos_format_date('short-date-time',$email['date' ])).
			' : </span> ';
		$out.='<span class="from"    >   '.ent($email['from'    ]).'</span> ';
		$out.='<span class="to"      > '.
			'<span class="right-arrow"></span> '.
			ent(mb_substr($email['to'],0,100)).'</span> ';
		$out.='</div>';
		$out.='<div  class="subject" >'.
			(trim($email['body'])!=='' ? '<span class="open-button"></span>' : '').
			($email['type']==='mail-import' ? '<a class="mail-import"  href="'.ent($email['url']).'" target="blank">'.ent('msg').'</a>' : '').
			ent($email['subject' ]).'</div> ';
		if(trim($email['body'])!=='')
		{
			$out.='<div class="body"    >   '.str_replace("\n","<br/>",ent($email['body'])).'</div> ';
		}
		$out.='</li>';
	}
	$out.='</ul>';
	$out.='</div>';
	return $out;
}


/**
 * Validate or actually send one or more emails from a event form.
 *
 * If $event is false then this function only does validation.
 */
function demosphere_event_edit_form_send_all_emails(&$form,$event=false,$validationData=false)
{
	// only validate/send if there are email addresses to which we want to send an email
	$emails=trim($form['event_email_address']['value']);
	if($emails==''){return;}

	$addresses=explode(',',$emails);
	foreach($addresses as $to)
	{
		$to=trim($to);
		if(filter_var($to, FILTER_VALIDATE_EMAIL)===false &&
		   !preg_match('/^[a-z0-9_-]+@[a-z0-9-]+$/i',$to))
		{
			$form['event_email_address']['error']=t('invalid email address:'.ent($to));
			continue;
		}
		demosphere_event_edit_form_send_single_email($to,$form,$event,$validationData);
	}
}

/**
 * Validate or actually send single email from a event form.
 *
 * If $event is false then this function only does validation.
 */
function demosphere_event_edit_form_send_single_email($to,&$form,$event=false,$validationData=false)
{
	global $user,$demosphere_config,$base_url;
	$values=dlib_array_column($form,'value');

	$emails=$demosphere_config['emails'];
	//var_dump($emails);
	// ********* we have to figure out which email to send ***
	$selectedMail=false;
	foreach($emails as $nb => $email)
	{
		$selectField='event_email_select_'.$nb;
		if($values[$selectField]!=0){$selectedMail=$nb;}
	}

	if($event===false && $selectedMail===false)
	{
		$form['event_emails']['error']=
					   t('No email selected. This should not happen. Please contact administrator.');
		return;
	}

	$subjectField='event_email_subject_'.$selectedMail;
	$subject = $values[$subjectField];

	$bodyField='event_email_body_'   .$selectedMail;
	$message = $values[$bodyField];


	// abort if trying to send first mail ("your event has been published")
	// and status is not published
	if($event===false && $selectedMail===0 && !$validationData['isPublished'])
	{
		$form['event_email_select_'.$selectedMail]['error']=
					   t('event is not published... this is incompatible with the email you are sending');
		return;
	}

	if($event===false && !$validationData['isPublished'] &&
	   (strpos($message,'%self-edit-link')!==false || 
		strpos($message,'%self_edit_password')!==false))
	{
		if($form['moderationStatus']['value']!=Event::moderationStatusEnum('waiting'))
		{
			$form['event_email_select_'.$selectedMail]['error']=
				t('The event is not published and moderation is not "waiting"... but the email you are sending contains a self-edit link. Self-editing unpublished, not "waiting" events is not possible.');
		}
		else
		{
			dlib_message_add(t('You have sent an email with a self-edit link for an event that is not published (but is "waiting"). The recepient of the email will be able to view and edit this event, but not publish it.'),'warning');
		}

		return;
	}

	if($event===false){return;}
	// ********** validation has finished here. 
	// ********** Now we have a valid event.

	$sender=demosphere_event_edit_form_email_sender();
	$from=$sender['email'];
	if($from===false || trim($from)===''){fatal('Sender email is empty or not set. This should not happen. Please contact admin.');}
	$message=demosphere_event_edit_form_email_string_replace($message,$event,true,$to);
	$subject=demosphere_event_edit_form_email_string_replace($subject,$event,true,$to);

	require_once 'lib/class.phpmailer.php';
	$mail=new PHPMailer();
	$mail->CharSet="utf-8";
	$mail->SetFrom($from);
	$mail->AddAddress($to);
	$mail->Subject=$subject;
	$mail->Body=$message;
	if($mail->Send())
	{
		dlib_message_add(t('sent mail "@title" to @to with subject:@subject',
						   ['@title'=>$emails[$selectedMail]['name'],
							'@to'=>$to,
							'@subject'=>$subject]));

		$message="[ ".$to." ]\n\n".$message;
		require_once 'demosphere-emails.php';
		demosphere_emails_notification_send('archive-sent',$subject,false,$message);

		// Save copy of mail into Message so that it appears in message list.
		if($demosphere_config['mail_import_imap_server']!==false)
		{	
			try
			{
				$m=Message::createFromRaw($mail->getSentMIMEMessage());
				$m->isSent=1;
				$m->finishedReading=time();
				$m->save();
			}catch(Exception $e){dlib_message_add('Saving copy of sent mail failed.','error');}
		}
	}
	else
	{
		dlib_message_add(t('failed to send mail "@title" to @to with subject:@subject',
						   ['@title'=>$emails[$selectedMail]['name'],
							'@to'=>$to,
							'@subject'=>$subject]),'error');
		
	}

	// Log email message 
	$c=changed_lines($values['event_email_body_'.$selectedMail],
					 $form  ['event_email_body_'.$selectedMail]['default-value']);
	if($c!==''){$c=preg_replace('@^@m','> ',$c);}
	$event->logAdd("sent mail $selectedMail (".
				   $emails[$selectedMail]['name'].")  to '$to'\n".
				   $c);
}

//! determine who is sending this mail (from:)
function demosphere_event_edit_form_email_sender()
{
	global $user,$demosphere_config;
	foreach($demosphere_config['email_senders'] as $s)
	{
		if($s['uid']===0                ){$defaultSender=$s;}
		if($s['uid']===intval($user->id)){return $s;}
	}
	return $defaultSender;
}

//! Replace tokens in email message or subject
//! Note: this is called twice, 
//! * before displaying the edit form, to replace any tokens that can be replaced (without having a complete event).
//! * after event save, where all event fields (like title, and time) are really ready
function demosphere_event_edit_form_email_string_replace($text,$event,$eventIsReady=false,$to=false)
{
	global $base_url,$demosphere_config;
	$sender=demosphere_event_edit_form_email_sender();

	if($eventIsReady)
	{
		require_once 'demosphere-date-time.php';
		// prepare event information that will be included in body
		$title=$event->title;
		$title=str_replace('_','',$title);
		$startTime=demos_format_date('full-date-time',$event->startTime);
		// FIXME: use demos_format_date
		$shortStartTime=strftime('%e %b %R',$event->startTime);
		$shortStartTime=str_replace(' 03:33','',$shortStartTime);

		$text=str_replace('%title'           ,$title         ,$text);
		$text=str_replace('%start_time'      ,$startTime     ,$text);
		$text=str_replace('%short_start_time',$shortStartTime,$text);
	}

	// event url related
	// FIXME should be self_edit_link not self-edit-link
	$text=str_replace('%self-edit-link','%event_url/edit?selfEdit=%self_edit_password',$text);
	$text=str_replace('%incomplete',t('For now, we have published this:').
						 "\r\n%event_url",$text);
	$text=str_replace('%event_url',$base_url.'%event_path',$text);
	$text=str_replace('%event_path','/'.$demosphere_config['event_url_name'].'/%event_id',$text);

	if($event->id)
	{
		$text=str_replace('%event_id'  ,$event->id ,$text);
	}

	// other
	$text=str_replace('%first_name',$sender['name'],$text);
	$text=str_replace('%site_name' ,$demosphere_config['site_name'],$text);
	$text=str_replace('%sitename'  ,$demosphere_config['site_name'],$text);
	$text=str_replace('%publishing_guidelines_url',Post::builtInUrl('publishing_guidelines'),$text);

	if($eventIsReady && strpos($text,'%self_edit_password')!==false) 
	{
		require_once 'demosphere-self-edit.php';
		$selfEdit=demosphere_self_edit_create($to,$event->id,$event->repetitionGroupId);
		$text=str_replace('%self_edit_password',$selfEdit['password'],$text);
	}
	return $text;
}

/** poorman's diff, quick hack.
 * FIXME: wait for some php library that provides the full algorithm.
 * Or use dlib/diff.php
 * @param $src a string containing "\n" serparated lines
 * @param $ref a string containing "\n" serparated lines
 * @return a string containing "\n" serparated lines that exist in $src but not in $ref
 */
function changed_lines($src,$ref)
{
	$res='';
	$srcLines=explode("\n",$src);
	$refLines=explode("\n",$ref);
	$refLines=array_flip($refLines);
	foreach($srcLines as $line)
	{
		if(!isset($refLines[$line])){$res.=$line."\n";}
	}
	return $res;
}

//! Confirmation form when deleting an event.
function demosphere_event_edit_form_delete($id)
{
	global $currentPage;
	// User wants to edit an existing event.
	$event=Event::fetch($id,false);
	if($event===null){dlib_not_found_404();}

	$currentPage->title=t('Delete: ').$event->title;

	if(isset($event->extraData['public-form']) &&
	   $event->created>time()-3600 &&
	   $event->getModerationStatus()==='trash')
	{
		return '<p>'.ent(t('This is an event that has just been created in the public form. Sombebody might still be editing it. You should not delete it.')).'</p>';
	}
	
	$items=[];
	$items['description']=
		['html'=>'<p>'.t('You can either move this event to the trash bin or really delete it.').'</p>'.
		 '<ul><li>'.t('Events in the trash bin can be recovered. They will be automatically deleted after one week.').'</li>'.
		 '<li>'.t('Deleted events can NOT be recovered.').'</li></ul>'];
	$items['trash']=
		['type'=>'submit',
		 'submit'=>function()use($event)
			{
				global $base_url;
				$event->trash();
				$event->save();
				dlib_redirect($event->url());
			},
		 'value'=>t('Move to trash'),];
	$items['delete']=
		['type'=>'submit',
		 'submit'=>function()use($event)
			{
				global $base_url;
				$event->delete();
				dlib_redirect($base_url);
			},
		 'value'=>t('Delete'),];
	$items['cancel']=
		['type'=>'submit',
		 'submit'=>function()use($event){dlib_redirect($event->url().'/edit');},
		 'value'=>t('Cancel'),];

	require_once 'dlib/form.php';
	return form_process($items);
}

?>