<?php

/** @defgroup mail_import Mail Import
 *  @{
 */


//! $mail_import_config : configuration options for mail_import.
//! List of $mail_import_config keys:
//! - cache_dir
//! - imap_inbox
//! - imap_server
//! - imap_password
//! - imap_username
//! - cache_time
//! - tmp_dir
//! - mbox_abs_path
//! - dir_url
//! -::-;-::-
$mail_import_config;


function mail_import_add_paths(array &$paths,&$options)
{
	require_once 'dcomponent/dcomponent-common.php';
	require_once 'demosphere-safe-domain.php';

	// Add moderator role paths generated by dbobject-ui interface for MailRule class 
	$dbui=preg_grep('@^dbobject-ui-.*-mailrule$@',array_keys($paths));
	foreach($dbui as $k){$paths[$k]['roles'][]='moderator';}

	$paths[]=['path'=>'@^mail-import/send-mail/?([0-9]+)?$@',
			  'roles' => ['admin','moderator'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_send_mail',];


	// When the user clicks on the big "finish" button of a message
	$paths[]=['path'=>'@^mail-import/message/(\d+)/finish$@',
			  'roles' => ['admin','moderator'],
			  'output'=>false,
			  'function'=>function($id)
		      {
				  $message=Message::fetch($id);
				  $message->finishedReading=val($_POST,'finish','true')==='true' ? time(): 0;
				  $message->save();
				  echo 'finished-ok-'.$id;
			  }
			 ];


	$paths[]=['path'=>'@^mail-import/message/(\d+)/display-raw$@',
			  'output'=>false,
			  'roles' => ['admin','moderator'],
			  'function'=>function($id)
		      {
				  $message=Message::fetch($id);
				  $raw=$message->getRawMail();
				  header('Content-Type: application/octet-stream');// should be "message/rfc822" but force download
				  header("Content-Disposition: attachment;filename=message-raw-".$id);
				  header("Content-Length: " .strlen($raw));
				  echo $raw;
			  }
			 ];

	$paths[]=['path'=>'@^mail-import/message/(\d+)/view-attachment@',
			  'output'=>false,
			  'access' => 'demosphere_safe_domain_check', 
			  'function'=>function($id){Message::fetch($id)->viewAttachment($_GET['part']);}
			 ];

	$paths[]=['path'=>'@^mail-import/message/(\d+)/import-to-demosphere@',
			  'output'=>false,
			  'roles' => ['admin','moderator'],
			  'function'=>function($id)
		      {
				  $message=Message::fetch($id);
				  dcomponent_import_to_demosphere($message->subject,
												  $message->getFullHtml(),
												  'message-'.$message->id);
			  }
			 ];

	$paths[]=['path'=>'mail-import',
			  'roles' => ['admin','moderator'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_message_list',
			 ];

	$paths[]=['path'=>'mail-import/mailing-lists',
			  'roles' => ['admin','moderator'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_mailing_lists',
			 ];

	$paths[]=['path'=>'@^mail-import/message/(\d+)/manage-rules@',
			  'roles' => ['admin','moderator'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_message_rule_manager',
			 ];

	$paths[]=['path'=>'@^mail-import/finish/(ordinary|old|thread)/?([0-9]+)?$@',
			  'roles' => ['admin','moderator'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_finish',
			 ];

	$paths[]=['path'=>'@^mail-import/message/(\d+)/display-details@',
			  'roles' => ['admin','moderator'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_message_details',
			 ];

	$paths[]=['path'=>'@^mail-import/message/(\d+)$@',
			  'roles' => ['admin','moderator'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_message',
			 ];

	$paths[]=['path'=>'@^mail-import/message/(\d+)/inside-frame@',
			  'output'=>false,
			  'access' => 'demosphere_safe_domain_check', 
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>'mail_import_view_message_inside_frame',
			 ];

	$paths[]=['path'=>'mail-import/mail-rule/ajax',
			  'roles' => ['admin','moderator'],
			  'output'=>'json',
			  'file' => 'mail-import/mail-import-view.php',
			  'function' => 'mail_import_mail_rule_ajax',
			 ];

	$paths[]=['path'=>'@^mail-import/imap/(\d+)/delete@',
			  'roles' => ['admin'],
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>function($uid)
		      {
				  require_once 'dlib/form.php';
				  $items['top']=['html'=>t('Are you sure you want to permanently delete this mail from imap server ?')];
				  $items['confirm']=['type'=>'submit',
									 'value'=>'confirm',
									 'redirect'=>'mail-import',
									 'submit'=>function()use($uid)
					  {
						  $imap=mail_import_imap_get_connection();
						  $ok=imap_delete($imap,$uid,FT_UID);
						  mail_import_imap_errors($ok);
						  dlib_message_add('message deleted');
						  mail_import_imap_errors(imap_expunge($imap));
					  },];
				  return form_process($items);
			  }
			 ];

	// This is necessary when using a Content-Security-Policy which doesn't allow for inline js.
	$paths[]=['path'=>'@^mail-import/js-config@',
			  'output'=>false,
			  'roles' => true, 
			  'file'=>'mail-import/mail-import-view.php',
			  'function'=>function()
		      {
				  global $demosphere_config,$base_url;
				  // Allow browser to cache this for 24 hours
				  // Note : these override headers already sent 
				  header('Expires: '.gmdate(DATE_RFC1123, time()+3600*24));
				  header('Cache-Control: public, max-age=86400');
				  header('Content-Type: text/javascript');

				  $out='';
				  $out.='demosphere_config='.dlib_json_encode_esc(['std_base_url'=>$demosphere_config['std_base_url']]).';';
				  $out.='base_url='.dlib_json_encode_esc($base_url).';';
				  $out.='trackLinkWarn='.dlib_json_encode_esc(t('This link might be tracking you. Do you want to continue ?')).";";
				  $out.='document.addEventListener("DOMContentLoaded", function()
                    {
                         document.querySelectorAll("a[data-tracking-href]").forEach(function(a)
                         {
                              var f;
                              a.addEventListener("click",f=function(e)
                              {
                                  var url=a.getAttribute("data-tracking-href");
                                  if(confirm(trackLinkWarn+"\n\n"+url+"\n"))
                                  {
                                       a.setAttribute("href",url);
                                       a.classList.remove("demosphere-tracker-link");
                                       a.removeEventListener("click",f);
                                  }
                              });
                         });
                    });';
				  echo $out;
			  },
			 ];
	  
}

function mail_import_cron()
{
	global $demosphere_config,$base_url;

	mail_import_imap_fetch();

	Message::rehighlightAll();

	Message::cleanupCachedFiles();

	mail_import_imap_archive();

	mail_import_search_reindex_cron();

	$ok=mail_import_imap_check_sync();
	if($ok!==true)
	{
		echo "mail_import_imap_check_sync failed!\n";
		require_once 'lib/class.phpmailer.php';
		$mail=new PHPMailer();
		$mail->CharSet="utf-8";
		$mail->SetFrom('root@localhost');
		$mail->AddAddress($demosphere_config['admin_email']);
		$mail->Subject='mail_import_imap_check_sync failed for '.$demosphere_config['site_name'];
		$mail->Body = "Mail import has detected that imap mails and database Messages are out of sync.\n".
			"You should read the output of your cron:\n".
			($_SERVER['SERVER_ADDR']==='178.33.109.191' ?
			 "grep -B 200 mail_import_imap_check_sync /var/log/demosphere/".$demosphere_config['site_id']."-cron.log\r\n"
			 :'');
		$mail->Send();
	}
}

//! Changes in mail rule can impact a large number of messages (MailRule::refreshMailRules()). 
//! In that case, the update is postponed to cron. 
function mail_import_search_reindex_cron()
{
	$r=variable_get('mail-import-search-reindex','');
	if($r===false){return;}
	
	$ct=0;
	foreach($r as $id)
	{
		$m=Message::fetch($id,false);
		if($m===null){continue;}
		$m->searchReindex();
		// memory leak in imap_fetchbody()
		$ct++;
		if(!($ct%100)){mail_import_imap_get_connection(false,$x,true);}
	}

	variable_delete('mail-import-search-reindex','');
}

// *****************************************************
// imap
// *****************************************************

//! Get new mail from imap, only if it has not been done since 10 minutes and it has not been explicitly requested.
function mail_import_imap_fetch_if_needed()
{
	global $mail_import_config;
	$last=variable_get('mail-import-last-fetched');
	$refetch=val($_GET,'refetch');
	$doFetch=false;
	if(!$last                     ){$doFetch=true;}
	if($refetch && $refetch>=$last){$doFetch=true;}
	if((time()-$last)>(60*10)	  ){$doFetch=true;}

	if($doFetch){mail_import_imap_fetch();}
}

//! Fetch new mails from imap $mailbox and create corresponding Message's
//! $mailbox defaults to $mail_import_config['imap_inbox'] (which is always 'inbox' on hosted sites)
function mail_import_imap_fetch($mailbox=false,$limit=20)
{
	global $mail_import_config,$base_url,$user;
	ini_set('max_execution_time', max(120,ini_get('max_execution_time')));

	if($mailbox===false){$mailbox=$mail_import_config['imap_inbox'];}

	$now=time();
	variable_set('mail-import-last-fetched','',$now);

	$imap=mail_import_imap_get_connection($mailbox);

	// imap_search returns false when empty or error, so we need to check if empty with imap_check first to avoid thinking false is an error. 
	$check=imap_check($imap);
	mail_import_imap_errors($check!==false);
	if($check->Nmsgs===0){return;}

	// The simplest and most reliable way to check for new mail is to compare the full list of all uids on imap and in db
	$imapUids=imap_search($imap,'ALL',SE_UID);
	mail_import_imap_errors(is_array($imapUids));

	$messageUids=db_one_col("SELECT imapUid FROM Message WHERE imapMailbox='%s'",$mailbox);

	$newUids=array_diff($imapUids,$messageUids);

	if(count($newUids)===0)
	{
		dlib_message_add(t('no new mail'));
		return;
	}

	// Now fetch overviews of newUids and create each Message
	$overviews=imap_fetch_overview($imap,implode(',',$newUids),FT_UID);
	mail_import_imap_errors(is_array($overviews));
	$ct=0;
	foreach($overviews as $overview)
	{
		if(($ct++)>=$limit){break;}
		// special case: skip deleted mail (this should almost never happen)
		if($overview->deleted){continue;}

		try{$message=new Message($imap,$overview->uid);}
		catch (Exception $e) 
		{
			if($e->getMessage()==='Message::__construct: aborting attempt to create a duplicate message')
			{
				$ok=imap_delete($imap,$overview->uid,FT_UID);
				mail_import_imap_errors($ok);
				$ok=imap_expunge($imap);
				mail_import_imap_errors($ok);
				dlib_message_add('Removed duplicate message','warning');
				continue;
			}
			require_once 'dlib/mail-and-mime.php';
			$out ='Create Message failed: '.ent($e->getMessage())."<br/>";
			$out.=' please contact system administrator<br/>';
			$out.='&nbsp;&nbsp;&nbsp;from:'.ent($overview->from   ).'<br/>';
			$out.='&nbsp;&nbsp;&nbsp;to:'  .ent(oval($overview,'to','')).'<br/>';
			$out.='&nbsp;&nbsp;&nbsp;subj:'.ent(dlib_mime_header_decode($overview->subject)).'<br/>';
			$out.='&nbsp;&nbsp;&nbsp;uid:'. ent($overview->uid    ).'<br/>';
			if(strtotime($overview->date)<$now-3600*24*7)
			{
				$out.='This mail has had errors for more than one week. It has been deleted.';
				$ok=imap_delete($imap,$overview->uid,FT_UID);
				mail_import_imap_errors($ok);
				$ok=imap_expunge($imap);
				mail_import_imap_errors($ok);
			}
			else
			if($user->checkRoles('admin'))
			{
				$out.='&nbsp;&nbsp;&nbsp;<a href="'.ent($base_url.'/mail-import/imap/'.$overview->uid.'/delete').'">delete it</a>';
			}
			dlib_message_add($out,'error');
			continue;
		}

		// Force caching (speedup display, after this function is called from cron)
		$message->getHighlighted();

		$out=t("finished fetching !msgLink",
			   ['!msgLink'=>'<a href="'.ent($message->url()).'">'.$message->id.'</a>']);
		dcomponent_progress_log($out);
	}
}


//! Returns an imap connection from a global pool of cached per-folder connections.
//! If the connection does not exist it is created with mail_import_imap_open()
function mail_import_imap_get_connection($mailbox=false,&$reindexed=false,$close=false)
{
	global $mail_import_config;
	static $connections=[];

	// Very special case: purge all connections to free memory
	// imap_fetchbody() has a memory leak. The imap connection needs to be closed to free that memory.
	// This is necessary when processing a large number of Message bodies.
	// However, closing imap connections here, will make any existing references to those connections invalid.
	// You need to make sure none of them exist before calling this.
	if($close!==false)
	{
		foreach($connections as $connection)
		{
			imap_close($connection);
		}
		$connections=[];
		return;
	}

	if($mailbox===false){$mailbox=$mail_import_config['imap_inbox'];}
	if(isset($connections[$mailbox])){return $connections[$mailbox];}
	$imap=mail_import_imap_open($mailbox,$reindexed);
	if($imap===false)
	{
		throw new Exception('mail_import_imap_get_connection: failed to open imap connection');
	}
	$connections[$mailbox]=$imap;
	return $imap;
}

//! Opens an imap connection, optionally to a specific mailbox.
//! It will also automatically detect a change in uidvalidity and reindex Messages.
//! Don't use this function directly, use mail_import_imap_get_connection()
function mail_import_imap_open($mailbox=false,&$reindexed=false)
{
	global $mail_import_config;
	if($mailbox===false){$mailbox=$mail_import_config['imap_inbox'];}

	// Very simple locking, to avoid concurrent actions by another process during long operations (rarely used)
	// If we need something more sophisticated, we 
	if($mailbox===$mail_import_config['imap_inbox'])
	{
		for($i=0;$i<10;$i++)
		{
			$lock=variable_get('mail-import-inbox-lock','');
			if($lock!==false && $lock<time()-60*10)
			{
				variable_delete('mail-import-inbox-lock','');
				$lock=false;
			}
			if($lock===false){break;}
			sleep(5);
		}
		if($lock!==false){throw new Exception('mail_import_imap_open: mailbox is locked');}
	}

	$imap=@imap_open($mail_import_config['imap_server'].$mailbox,
					 $mail_import_config['imap_username'],
					 $mail_import_config['imap_password']);
	mail_import_imap_errors();
	if($imap===false){return false;}

	$status=imap_status($imap,$mail_import_config['imap_server'].$mailbox,SA_UIDVALIDITY);
	mail_import_imap_errors();
	if($status===false){return false;}

	$mailboxDesc=variable_get($mailbox,'mail-import-mailbox');
	if($mailboxDesc===false || $mailboxDesc['uidvalidity']!=$status->uidvalidity)
	{
		$reindexed=true;
		mail_import_imap_reindex_mailbox($imap);
		if($mailboxDesc===false){$mailboxDesc=[];}
		$mailboxDesc['uidvalidity']=$status->uidvalidity;
		variable_set($mailbox,'mail-import-mailbox',$mailboxDesc);
	}

	return $imap;
}

//! Queries an imap connection to get the current mailboxe's name (Example: 'inblox')
function mail_import_imap_current_mailbox_name($imap)
{
	$check=imap_check($imap);
	mail_import_imap_errors($check!==false);
	return explode('}',$check->Mailbox)[1];
}

//! Update all uids of Messages that have the same mailbox that the current mailbox of the $imap connection.
function mail_import_imap_reindex_mailbox($imap)
{
	$mailbox=mail_import_imap_current_mailbox_name($imap);

	variable_set('mail-import-inbox-lock','',time());

	// Reset uids in Messages
	db_query("UPDATE Message SET imapUid=0 WHERE imapMailbox='%s'",$mailbox);

	$check=imap_check($imap);
	mail_import_imap_errors($check!==false);
	if($check->Nmsgs==0){variable_delete('mail-import-inbox-lock','');return;}

	$uids=imap_search($imap,'ALL',SE_UID);
	mail_import_imap_errors($uids!==false);

	// Foreach uid in imap, find Message having same headers in db and update its uid and mailbox.
	foreach($uids as $uid)
	{
		$headers=imap_fetchheader($imap,$uid,FT_UID);
		mail_import_imap_errors($headers!==false);
		db_query("UPDATE Message SET imapMailbox='%s', imapUid=%d WHERE headersHash='%s'",$mailbox,$uid,hash('SHA256',$headers));
	}
	variable_delete('mail-import-inbox-lock','');
}

//! Returns decoded / fixed versions of imap overview fields for a single message.
function mail_import_imap_fetch_overview($imap,$uid)
{
	$overviews=imap_fetch_overview($imap,$uid,FT_UID);
	mail_import_imap_errors(is_array($overviews));
	$res=[];
	foreach(['subject','from','to'] as $field)
	{
		if(!isset($overviews[0]->$field)){$res[$field]=false;continue;}
		$value=$overviews[0]->$field;
		require_once 'dlib/mail-and-mime.php';
	 	$value=dlib_mime_header_decode($value);
		if($field==='subject'){$value=dlib_cleanup_plain_text($value);}
		$res[$field]=$value;
	}
	$res['date']=isset($overviews[0]->date) ? strtotime($overviews[0]->date) : false;
	return $res;
}


//! Don't keep all mails in same imap inbox folder.
//! It will become too large and be a problem for server backups.
//! Move mails to archive mailbox when:
//! They are large (>50Mb)
//! They are more than 1 month old
//! They all fit on nice month boundaries 
function mail_import_imap_archive()
{
	global $mail_import_config;
	$minArchiveSize=50*1000*1000;

	$imap=mail_import_imap_get_connection();

	// we need to check to avoid errors further on
	$check=imap_check($imap);
	mail_import_imap_errors($check!==false);
	if($check->Nmsgs===0){return;}

	// Fetch mail sizes from overviews
	$overviews=imap_fetch_overview($imap,'1:*');
	mail_import_imap_errors(is_array($overviews));
	$sizes=[];
	foreach($overviews as $overview){$sizes[$overview->uid]=$overview->size;}
	
	$oneMonthAgo=strtotime('1 month ago');

	$totSize=0;
	$messagesInfo=db_arrays("SELECT id,imapUid,dateFetched FROM Message WHERE imapMailbox='%s' ORDER BY dateFetched ASC",
							$mail_import_config['imap_inbox']);
	if(count($messagesInfo)===0){return;}
	$firstDate=min(dlib_array_column($messagesInfo,'dateFetched'));
	$lastDate =max(dlib_array_column($messagesInfo,'dateFetched'));
	$endMonth=false;
	$breakDate=false;
	foreach($messagesInfo as $messageInfo)
	{
		$uid =$messageInfo['imapUid'];
		$date=$messageInfo['dateFetched'];
		$totSize+=$sizes[$uid];
		// did not find enough mail to archive
		if($date>$oneMonthAgo){return;}
		// total size is not yet enough to archive
		if($totSize<$minArchiveSize){continue;}
		// ok, we have enough mail to archive, now wait for next month boundary to have mailboxes on clean month boundaries
		$month=date('Y-m',$date);
		if($endMonth===false){$endMonth=$month;}
		if($month!==$endMonth){$breakDate=$date;break;}
	}
	if($breakDate===false){return;}

	$beginMonth=date('Y-m',$firstDate);
	$newBoxName='inbox-'.$beginMonth.'-'.$endMonth;

	echo "Archiving ".$totSize."b to new mailbox: ".$newBoxName."\n";

	$ok=imap_createmailbox($imap,$mail_import_config['imap_server'].$newBoxName);
	mail_import_imap_errors($ok);

    // Move one message at a time to avoid huge problems that might happen if something fails during this long operation
	$ct=1;
	foreach($messagesInfo as $messageInfo)
	{
		$id  =$messageInfo['id'];
		$uid =$messageInfo['imapUid'];
		$date=$messageInfo['dateFetched'];
		if($date>=$breakDate){break;}
		$ok=imap_mail_move($imap,$uid,$newBoxName,CP_UID);
		mail_import_imap_errors($ok);
		db_query("UPDATE Message SET imapMailbox='%s',imapUid=0 WHERE id=%d",$newBoxName,$id);
		if((($ct++)%200)===0){imap_expunge($imap);}
	}
	$ok=imap_expunge($imap);
	mail_import_imap_errors();

	// Now actually set uids. get_connection calls open that will automatically call reindex.
	$newImap=mail_import_imap_get_connection($newBoxName);
	// so this explicit call to reindex is just in case somebody accessed archived mail during transfer (which can be long)
	mail_import_imap_reindex_mailbox($newImap);
	echo "Archive finished\n";
}

//! This function checks if all messages are in sync with all mailboxes
function mail_import_imap_check_sync()
{
	global $mail_import_config;
	$imap=mail_import_imap_get_connection();

	$ok=true;

	// *** Check if mailboxes are identical on imap and in Message's

	// build list of imap mailboxes
	$imapBoxes=imap_list($imap,$mail_import_config['imap_server'].$mail_import_config['imap_inbox'],'%');
	mail_import_imap_errors(is_array($imapBoxes));
	foreach($imapBoxes as &$imapBox){$imapBox=strtolower(explode('}',$imapBox)[1]);}
	unset($imapBox);
	sort($imapBoxes);

	// build list of Message mailboxes

	$dbMboxes=db_one_col("SELECT imapMailbox FROM Message GROUP BY imapMailbox");
	sort($dbMboxes);
	
	if($imapBoxes!==$dbMboxes)
	{
		// special case: empty mailbox, never used
		if(count($dbMboxes)===0 && $imapBoxes==[$mail_import_config['imap_inbox']])
		{
			$check=imap_check($imap);
			mail_import_imap_errors(is_object($check));
			if($check->Nmsgs==0){return true;}
		}

		echo "Mailboxes list does not match\n";
		vd($imapBoxes,$dbMboxes);
		return false;
	}

	// *** Check if list of mails is identical on imap and in Message's
	foreach($dbMboxes as $mailbox)
	{
		// Don't use mail_import_imap_get_connection() because we need to close to avoid hitting max connections limit
		$mboxImap=mail_import_imap_open($mailbox);
		if($mboxImap===false){throw new Exception('mail_import_imap_check_sync: failed to open imap connection');}

		$imapUids=imap_search($mboxImap,'ALL',SE_UID);
		mail_import_imap_errors();
		// imap does not distinguish between fail and empty search :-(
		if($imapUids===false){$imapUids=[];}
		sort($imapUids);

		$dbUids=db_one_col("SELECT imapUid FROM Message WHERE imapMailbox='%s'",$mailbox);
		$dbUids=array_map('intval',$dbUids);
		sort($dbUids);
		
		// Special case for inbox: ignore unfetched mail
		if($mailbox==='inbox')
		{
			$unfetched=array_diff($imapUids,$dbUids);
			if(count($unfetched)<20)
			{
				$imapUids=array_values(array_diff($imapUids,$unfetched));
				sort($imapUids);
			}
		}
		
		if($imapUids!==$dbUids)
		{
			echo "Mailbox: ".$mailbox." : list of uids is different in imap and in Message db table\n";
			echo "uids in imap not present in Message:\n";
			vd(array_diff($imapUids,$dbUids));
			echo "uids in Message not present in imap:\n";
			vd(array_diff($dbUids,$imapUids));
			echo "Duplicate uids in Message:\n";
			vd(array_filter(array_count_values($dbUids),function($v){return $v>1;}));
			$ok=false;
		}
		imap_close($mboxImap);
	}
	return $ok;
}


//! Create a fake imap connection to a raw mail data string.
//! Warning: this is no longer explicitly supported by the php imap functions (but works).
function mail_import_imap_open_raw($rawMessage)
{
	global $mail_import_config;
	// IMAP open uses path from home dir :-( and refuses ../.. paths
	// so the only reliable method is to change the HOME env variable
	$dir=$mail_import_config['mbox_abs_path'];
	$fullFname=tempnam($dir,'tmp-mime-message-');
	if($fullFname===false){throw new Exception("mail_import_imap_open_raw: can't create tmp file in ".ent($dir));}
	$dir  =dirname ($fullFname);
	$fname=basename($fullFname);
	if(file_put_contents($fullFname,$rawMessage)===false){throw new Exception("mail_import_imap_open_raw: failed to write to tmp file");}
	$origHome=getenv('HOME');
	putenv('HOME='.$dir);
	$imap=imap_open($fname,'','',OP_READONLY);
	mail_import_imap_errors($imap!==false);
	putenv('HOME'.($origHome===false ? '' : '='.$origHome));
	return [$imap,$fname];
}


//! Imap generates warnings and errors, which are stored internally and generate annoying PHP errors and warnings when program ends.
//! We deal with them here, to avoid messages in Apache error logs.
function mail_import_imap_errors($ok=true,$errorMessage='')
{
	global $dlib_config;
	$allProblems=[];
	$errors=imap_errors();
	if($errors===false){$errors=[];}
	foreach($errors as $error)
	{
		if($error==='SECURITY PROBLEM: insecure server advertised AUTH=PLAIN'){continue;}
		// Undisclosed recipients
		if(strpos($error,'Unterminated mailbox'      )===0){continue;}
		if(strpos($error,'Unexpected characters at end of address')===0){continue;}
		// Bad mime types are handled later by us
		if(strpos($error,'Unknown MIME type')===0){continue;}
		// Missing quotes in attachment filename (there maybe other reasons?)
		if(strpos($error,'Unexpected characters at end of parameters')===0){continue;}
		// Bad email addresses
		if(strpos($error,'Must use comma to separate addresses')===0){continue;}
		// Invalid email addresses
		if(strpos($error,'Invalid mailbox list')===0){continue;}
		// Unknown cause
		if($error==='Missing parameter'){continue;}
		dlib_log('Imap error: '.$error);
		$allProblems[]=$error;
	}

	$alerts=imap_alerts();
	if($alerts===false){$alerts=[];}
	foreach($alerts as $alert)
	{
		dlib_log('Imap warning: '.$alert);
		$allProblems[]=$alert;
	}

	if(count($allProblems))
	{
		$details='';
		if($dlib_config['debug'])
		{
			$stack=debug_backtrace();
			$details='<br/>'.ent(implode('; ',$allProblems)).'<br/>';
			$details.=ent($stack[0]['file'].':'.$stack[0]['line'].' '.$stack[1]['function'].'()');
		}
		dlib_message_add(t('Problem while processing mail (Imap).').$details,'warning');
	}

	if(!$ok)
	{
		throw new Exception('Imap error. '.$errorMessage.($dlib_config['debug'] ? ' :: '.ent(implode('; ',$allProblems)) : '' ));
	}

	return count($allProblems)>0;
}

function mail_import_imap_test()
{
	$imap=mail_import_imap_get_connection();
	//vd(imap_fetchbody($imap,100000000,FT_UID));
	//imap_delete($imap,15821,FT_UID);
	//$overviews=imap_fetch_overview($imap,'1:*');
	//vd($overviews);
}


// *****************************************************
// text matching
// *****************************************************

function mail_import_text_matching_hook($op,$arg1=null)
{
	require_once 'dcomponent/dcomponent-common.php';
	global $mail_import_config,$currentPage,$base_url;

	// operations that don't need full $message
	switch($op)
	{
	case 'allTids': return db_one_col("SELECT id FROM Message WHERE dateFetched>%d",time()-3600*24*7*2);
	case 'displayListInit':
		$currentPage->addCssTpl('demosphere/mail-import/mail-import.tpl.css');
		$currentPage->addCssTpl('demosphere/mail-import/message-item.tpl.css');
		$currentPage->addCssTpl('dlib/dropdown.css');
		$currentPage->addJs('dlib/dropdown.js');
		$currentPage->addJs('demosphere/mail-import/mail-import.js');
		return;
	case 'isExpired':
		$dateFetched=(int)db_result_check("SELECT dateFetched FROM Message WHERE id=%d AND importance!='trash'",$arg1);
		return $dateFetched < time()-3600*24*7*2;
	case 'matchInfo': 
		global $base_url;
		$messageData=db_array("SELECT ((importance='ordinary' AND isHighlighted=0) OR finishedReading!=0) AS isOld ,subject FROM Message WHERE id=%d",$arg1);
		if($messageData===null){return [];}
		return ['class'=> $messageData['isOld'] ? 'old ' :'recent ',
				'title'=>$messageData['subject'],
				'url'  =>$base_url.'/mail-import/message/'.intval($arg1),
			   ];
	}

	$message=Message::fetch($arg1,false);
	if($message!==null && $message->importance==='trash'){$message=null;}

	switch($op)
	{
	case 'getContents':
		if($message===null){return false;}
		return [$message->getFullHtml(true,true),'html'];
	case 'displayInList': 
		if($message===null){return '<p>message '.intval($arg1).' does not exist</p>';}
		require_once 'mail-import-view.php';
		require_once 'dlib/template.php';
		$vars=['item'=>$message,
			   $message->viewData(),
			   'scriptUrl'=>$base_url.'/mail-import'];
		$parts=template_render_parts('message-item.tpl.php',$vars);
		echo template_render('dcomponent/dcitem.tpl.php',array_merge($vars,$parts));
		return;
	}
}



//! Returns a list of email addresses for which this mailbox is a destination.
//! This only works for hosted domains.
function mail_import_destination_addresses()
{
	global $demosphere_config;
	require_once 'demosphere-emails.php';
	$domain=demosphere_emails_hosted_domain();
	if($domain===false){return [];}

	// Fetch all aliases
	$mailboxAddress=$demosphere_config['site_id'].'-mailbox@'.$domain;
	$aliases=demosphere_emails_hosted_fetch_data()['aliases'];

	// Build an index dest=>alias
	$parents=[];
	foreach($aliases as $address=>$alias)
	{
		foreach($alias['dests'] as $dest){$parents[$dest][]=$address;}
	}

	// Recursively search the index for aliases that match a dest
	$mailboxDestinations=[];
	$searchRecursive=false;
	$searchRecursive=function($search,$depth=0)use($parents,&$mailboxDestinations,&$searchRecursive)
		{
			if(!isset($parents[$search])){return;}
			if($depth>20){fatal('Circular alias!');}
			foreach($parents[$search] as $parent)
			{
				// special case: mailbox is also an alias
				if($parent===$search){continue;}
				$mailboxDestinations[]=$parent;
				$searchRecursive($parent,$depth+1);
			}
		};
	$searchRecursive($mailboxAddress);
	return $mailboxDestinations;
}

//! Determines whether a url is an email link tracker.
//! This uses simple heuristics that are far from 100% reliable. But is still useful.
function mail_import_url_is_tracker($url)
{
	global $demosphere_config;
	static $domain=false;
	if(strpos($url,$demosphere_config['safe_base_url'])===0){return false;}
	if($domain===false)
	{
		require_once 'demosphere-emails.php';
		$domain=demosphere_emails_hosted_domain();
	}
	$parsed=parse_url($url);

	// Links that contain email of this domain are trackers
	if(strpos($url,'@'.$domain)!==false){return true;}

	// use invalid decoded url for better matching, length
	$durl=urldecode($url);
	// Black list certain words
	if(preg_match('@(track|\bclic|/lnk/|mailchimp|mailchi.mp|optiextension|[\./]us[0-9]+\.|list-manage|newsletter|sendgrid|uid=)@',$url)){return true;}

	// White list domains (common mainstream press links)
	// FIXME: this works only for France and is not maintainable.
	if(preg_match('@(lemonde\.fr|attac\.org|liberation\.fr|mediapart\.fr|marianne\.net|huffingtonpost\.fr|leparisien\.fr|20minutes\.fr|nouvelobs\.com)$@',
	 			  $parsed['host'])){return false;}

	// Article title in url (lots of mainstream press do this and get long urls)
	$durl=preg_replace('@(\p{L}{1,15}([_ -]))(\p{L}{1,15}\2){2,}\p{L}{1,15}@u','',$durl);
	
	return mb_strlen($durl)>95;
}

//! Displays urls to helps evaluating how well mail_import_url_is_tracker() is working.
function mail_import_url_is_tracker_test()
{
	$ids=db_one_col("SELECT id FROM Message ORDER BY dateFetched DESC LIMIT 1000");
	foreach($ids as $id)
	{
		$m=Message::fetch($id);
		$html=$m->getFullHtml();
		preg_replace_callback('@(<a [^>]*href=[\'"])(https?://[^\'"]*)([\'"])@sU',function($matches)
		{
			$url=dlib_html_entities_decode_all($matches[2]);
			$isTracker=mail_import_url_is_tracker($url);
			echo ($isTracker ? 'tr' : 'ok').':'.$url."\n";
		},$html);
	}
}



// *****************************************************
// Experimental: wordlist removal 2016/08. Remove this after a while if it is not used
// *****************************************************

// see also: dlib/wordlists.php

// Where should it be used ?
// Display: grey words out (Message:getFullHtml Article::fetchAndCleanupContent)
// text-matching : remove those words
// searchIndex   : remove those words


//function common_words_cron()
//{
// 	// foreach mailing list, foreach "from", for each feed (others?)
// 	//     check if it needs refresh (every week?)
// 	//     delete old common words
// 	//     determine new common words
//    //     save them
// 
//}
// 
//function mail_import_common_text($type,$matchStr)
//{
// 	$q="SELECT id FROM Message WHERE importance!='trash' ";
// 	switch($type)
// 	{
// 	case 'list':
// 		$q.=" AND listId='%s' ";
// 		break;
// 	case 'from':
// 		$q.=" AND `from` LIKE '%%%s%%' ";
// 		break;
// 	}
// 
// 	//$q.=" LIMIT 100";
// 
// 	$ids=db_one_col($q,$matchStr);
// 	$common=[];
// 	foreach($ids as $n=>$id)
// 	{
// 		vd($id);
// 		$message=Message::fetch($id);
// 		$text=$message->getHighlighted(true);
// 		if(strlen($text)>100000){continue;}
// 		echo "got text:".strlen($text)."\n";
// 		list($words,$positions)=dlib_simplify_text_with_pos($text,true);
// 		echo "textlength:".strlen($text)." simple:".count($words)."\n";
// 		if($n==0){$common[]=$words;continue;}
// 		$common=wordlist_find_common_between_set_and_doc($common,$words,3);
// 		vd($common);
// 		if(count($common)>2 && $n>10){die('pp');}
// 		if(count($common)===0){break;}
// 	}
// 
// 	$commonWordLists=[
// 		'type'=>'cwords-mail-import-list',
// 		'name'=>'<example mailing list id>',
// 
// 		'wordLists'=>[['xx','yy'],['aa','bb']],
// 		'date'=>12345,
// 		];
// 
//}


// *****************************************************
// install and config
// *****************************************************

function mail_import_install()
{
	require_once 'mail-import/Message.php';
	require_once 'mail-import/MailRule.php';
	
	Message::createTable();
	MailRule::createTable();

}

/** Check if config is ok, this is called during install and from cron.
 * @see http://api.drupal.org/api/function/hook_requirements
 * @param phase can be 'install' or 'runtime'. Some checks are different depending on phase.
 * @param the array that will be filled 
 */
function mail_import_config_check($phase,&$req)
{
	if($phase==='install'){return;}
	global $mail_import_config;
	require_once 'dcomponent/dcomponent-common.php';
    dcomponent_config_check_file('mail-import:cache_dir',$req,
								 $mail_import_config['cache_dir']);

	if(val($mail_import_config,'imap_server')===false)
	{
		_req_error('mail-import-imap',$req,'mail import imap not configured');
	}

}

function mail_import_config_check_fast()
{
	global $mail_import_config;
	if(!isset($mail_import_config)){fatal("mail_import_config_check_fast: no config!");}

	if(val($mail_import_config,'imap_server')===false)
	{fatal("mail_import_config_check_fast: imap_server");}

	if(!isset($mail_import_config['cache_dir']))
	{fatal("mail_import_config_check_fast: cache_dir not set");}
}

/** @} */

?>