<?php
/** @defgroup page_watch Page Watch
 *  @{
 */

//! $page_watch_config : configuration for page_watch.
//! List of $page_watch_config keys:
//! - log_file
//! -::-;-::-
$page_watch_config;


function page_watch_add_paths(&$paths,&$options)
{
	require_once 'dcomponent/dcomponent-common.php';

	// Add moderator role paths generated by dbobject-ui interface for Page class
	$dbui=preg_grep('@^dbobject-ui-.*-page$@',array_keys($paths));
	foreach($dbui as $k){$paths[$k]['roles'][]='moderator';}

	// **** self rendered pages

	$paths[]=['path'=>'@^page-watch(/view-changes)?$@',
			  'roles' => ['admin','moderator'],
			  'file'=>'page-watch/page-watch-view.php',
			  'function'=>'page_watch_view_changes',
			 ];

	$paths[]=['path'=>'@^page-watch/page/(\d+)/finish$@',
			  'output'=>false,
			  'roles' => ['admin','moderator'],
			  'function'=>function($id)
		{
			$page=Page::fetch($id);
			$page->setLastChecked(intval($_GET['t1']));
			$page->save();
			echo "finished-ok-".$id;
		}
			 ];

	$paths[]=['path'=>'page-watch/page/update-all',
			  'output'=>false,
			  'roles' => ['admin','moderator'],
			  'function'=>function(){Page::updatePages();}
			 ];

	// FIXME: this doesnt seem to be used anywhere
	$paths[]=['path'=>'@^page-watch/page/(\d+)/update@',
			  'output'=>false,
			  'roles' => ['admin','moderator'],
			  'function'=>function($id){Page::fetch($id)->update();}
			 ];

	$paths[]=['path'=>'page-watch/page/test-page',
			  'output'=>false,
			  'roles' => ['admin','moderator'],
			  'function'=>function(){Page::testPage();}
			 ];

	$paths[]=['path'=>'page-watch/page/create-test-scenario',
			  'output'=>false,
			  'roles' => ['admin','moderator'],
			  'function'=>function(){Page::createTestScenario();}
			 ];

	// **** std pages

	$paths[]=['path'=>'@^page-watch/page/(\d+)/display-html-diffs@',
			  'roles' => ['admin','moderator'],
			  'file'=>'page-watch/page-watch-view.php',
			  'function'=>function($id)
		{
			$page=Page::fetch($id);
			$t0=intval($_GET['t0']);
			$t1=intval($_GET['t1']);
			return page_watch_view_page_html_diffs($page,$t0,$t1);
		}
			 ];

	$paths[]=['path'=>'page-watch/admin',
			  'roles' => ['admin','moderator'],
			  'file'=>'page-watch/page-watch-view.php',
			  'function'=>'page_watch_view_page_watch_admin'
			 ];

	$paths[]=['path'=>'@^page-watch/page/(\d+)/configure-content-selector$@',
			  'roles' => ['admin','moderator'],
			  'function'=>function($id)
		{
			$page=Page::fetch($id);
			dcomponent_progress_log_start();
			$editorUrl=page_watch_create_html_selector_editor($page);
			dcomponent_progress_log_end(['redirect'=>$editorUrl,'boxDisplay'=>'fold']);
		},
			 ];



	// **** safe domain

	$paths[]=['path'=>'@^page-watch/page/(\d+)/display-highlighted-html@',
			  'output'=>false,
			  'access' => 'demosphere_safe_domain_check', 
			  'file'=>'page-watch/page-watch-view.php',
			  'function'=>function($id)
		{
			$page=Page::fetch($id);
			$t0=intval($_GET['t0']);
			$t1=intval($_GET['t1']);
			echo $page->getHighlightedHtml($t0,$t1);
		}
			 ];

	$paths[]=['path'=>'@^page-watch/page/(\d+)/display-revision@',
			  'output'=>false,
			  'access' => 'demosphere_safe_domain_check', 
			  'file'=>'page-watch/page-watch-view.php',
			  'function'=>function($id)
		{
			$page=Page::fetch($id);
			$t0=intval($_GET['t0']);
			echo $page->getRevisionHtml($t0);
		}
			 ];

}


function page_watch_cron()
{
	Page::updatePages();
	Page::cleanupOld();
}

// Adds $higlightOpen and $highlightClose (html tags) arround differences 
// in two html files.
// It also looks for future dates,times and keywords inside the diffs.
// We suppose both files have been parsed with tidy (with indentation and no wrap)
function highlight_html_diffs($html0,$html1)
{
	require_once 'dlib/html-tools.php';
	require_once 'dlib/diff.php';
	$highlightOpen='<cite class="demosphere_highlight_diff">';
	$highlightClose='</cite>';
	//file_put_contents('/tmp/h0',$html0);
	//file_put_contents('/tmp/h1',$html1);
	// only highlight changes in body
	$headPos0=strpos($html0,'<body');
	$headPos1=strpos($html1,'<body');
	$head0=substr($html0,0,$headPos0);
	$head1=substr($html1,0,$headPos1);
	$body0=substr($html0,$headPos0);
	$body1=substr($html1,$headPos1);
	
	// remove all attributes from html beafore diff, to avoid false diffs
	$r=[];
	$ls=explode("\n",$body0);
	foreach($ls as $l){$r[]=preg_replace('@(<[a-z0-9]+)\b[^>]*>@','$1>',$l);}
	$cleanBody0=implode("\n",$r);

	$r=[];
	$ls=explode("\n",$body1);
	foreach($ls as $l){$r[]=preg_replace('@(<[a-z0-9]+)\b[^>]*>@','$1>',$l);}
	$cleanBody1=implode("\n",$r);

	//$cleanBody1=$body1;$cleanBody0=$body0;

	$diffs=unix_diff($cleanBody0,$cleanBody1);
	$htmlLines=explode("\n",$body1);
	$markerOpen=" JHKH3476976HJBV ";
	$markerClose=" J24JHLKJ24MHMIHD ";
	$res=['timestamps'=>[],'keywordMatches'=>[],'diffBlocks'=>0];
	//echo "XXXX html0:\n".$html0."\n";
	//echo "XXXX html1:\n".$html1."\n";
	//var_dump($htmlLines);
	//var_dump($diffs);
	//echo "unclean:\n";
	//var_dump($htmlLines);
	//echo "clean:\n";
	//var_dump(explode("\n",$cleanBody1));

	// each diff covers a range of lines
	foreach($diffs as $diff)
	{
		$range=$diff['range'];
		if($range['action']=='d'){continue;}
		$start=$range['start-1'];
		$end  =$range['end-1'];
		// collect a range of lines in a block
		// so that we can then highlight dates and times in it
		$block='';
		for($lineNumber=$start-1;$lineNumber<=$end-1;$lineNumber++)
		{
			$line=$htmlLines[$lineNumber];
			$htmlLines[$lineNumber]='';
			// don't highlight lines with just an html tag
			if(preg_match('@^[ \t]*<[^>]*>[ \t]*$@',$line))
			{
				$block.=$line."\n";
				continue;
			}
			//echo "line $lineNumber:";var_dump($line);
			$block.=add_inline_tag_to_html($line,'cite',$markerOpen,$markerClose)."\n";
		}
		$block=str_replace($markerOpen."".$markerClose,'',$block);
		$block=str_replace($markerOpen ,$highlightOpen ,$block);
		$block=str_replace($markerClose,$highlightClose,$block);
		//echo "before dates:".$block."\n";
		// FIXME date&time <em>s can get very mixed up with diff <em>s
		require_once 'demosphere-common.php';
		$hl=highlight_future_dates_and_keywords($block,true,false);
		//var_dump($hl);
		//echo "date:".date('r',$hl['timestamps'][0])."\n";
		$block=$hl['content'];
		//echo "after  dates:".$block."\n";
		$htmlLines[$start-1]=$block;
		$res['timestamps']=array_merge($res['timestamps'],$hl['timestamps']);
		$res['keywordMatches']=array_merge($res['keywordMatches'],$hl['keywordMatches']);
	}
	$content=implode("\n",$htmlLines);
	// remove highlighting in <option>
	$content=preg_replace('@(<option[^<]*)<cite[^>]*>@','$1',$content);
	$content=preg_replace('@(<option[^<]*)<em[^>]*>@'  ,'$1',$content);
	$content=preg_replace('@(<option[^<]*)</em>@'      ,'$1',$content);
	$content=preg_replace('@(<option[^<]*)</cite>@'    ,'$1',$content);
	$res['content']=$head1.''.$content;
	$res['diffCount']=substr_count($content,$highlightOpen);
	return $res;
}

function add_inline_tag_to_html($text,$tagName,$begin,$end)
{
	static $inline='(var|u|tt|textarea|sup|sub|strong|strike|span|small|select|samp|s|q|label|kbd|input|img|i|font|em|dfn|code|br|big|bdo|basefont|b|acronym|abbr|a)';

	//echo "input :".$text."\n";
	$res=$begin.''.str_replace([     '<','>'],
							   [$end.'<','>'.$begin],$text).''.
		$end;
	
	$res=preg_replace('@'.$begin.'([ \t]*)'.$end.'@','$1',$res);
	// The only tag that has intersesting contents here is img
	$res=preg_replace('@(<img\b[^>]*>)@',$begin.'$1'.$end,$res);

	// Cleanup unneeded tags around an inline element (with no children)
	//echo 'bef    :'.$res."\n";
	$res=preg_replace('@'.$end.'(<'.$inline.'\b[^>]*>)'.$begin.'([^<]*)'.$end.'(</\2>)'.$begin.
					  '@',//$begin.'([^<]*)'.$end.'(</$2>)'.$begin.'@',
					  '$1$3$4',$res);
	//echo 'aft    :'.$res."\n";

	// Cleanup unneeded tags around an inline element (with one child)
	//echo 'bef    :'.$res."\n";
	//$res=preg_replace('@'.$end.'(<'.$inline.'\b[^>]*>)'.$begin.
	// 				  '([^<]*<'.$inline.'\b[^>]*>[^<]*</\4>[^<]*)'.
	// 				  $end.'(</\2>)'.$begin.
	// 				  '@',
	// 				  '$1$3$5',
	// 				  $res);
	//echo 'aft    :'.$res."\n";

	//echo 'res   :'.$res."\n";
	return $res;
	//$res=$text;
	//$res=preg_replace('@<(?!('.$inline.'\b))',$end.'<',$res);
	////	$res=preg_replace('@>(?!('.$inline.'\b))',$end.'<',$res);
	//return $begin.''.array('<'     ,'>'       ),
	// 							 array($end.'<','>'.$begin),$text).''.
	// 	$end;

}

function page_watch_tmp_test_op()
{
}

function page_watch_create_html_selector_editor($page)
{
	global $base_url,$html_selector_config;
	require_once 'html-selector/html-selector.php';
	require_once 'dlib/html-tools.php';

	$editor=HtmlSelectorEditor::create($page->htmlContentSelector,[$page->url],
									   ['require_once'=>'page-watch/page-watch.php',
										'function'=>'page_watch_html_selector_finished_callback',
										'data'=>$page->getId()],
									   ['title'=>$page->getName(),
										'display-callback' =>['require_once'=>'page-watch/page-watch-view.php',
															  'function'=>'page_watch_view_html_selector_callback',
															  'data'=>$page->getId()],
									   ]);

	$editor->save();
	$editorUrl=$base_url.'/html-selector/editor/'.$editor->id;
	return $editorUrl;
}

function page_watch_html_selector_finished_callback($pageId,$selector)
{
	require_once 'dlib/html-tools.php';
	global $base_url;
	$pageId=intval($pageId);
	$page=Page::fetch($pageId);
	$page->htmlContentSelector=$selector;
	$page->save();
	dlib_message_add('Html content selector successfully saved.');
	dlib_redirect($base_url.'/page-watch/page/'.$pageId);
}


function page_watch_install()
{
	require_once 'Page.php';
	Page::createTable();
	//var_dump($r);

	//echo "create table page_revision<br/>\n";
	db_query("DROP TABLE IF EXISTS page_revision");
	$r=db_query("CREATE TABLE page_revision (
  `id` int(11) NOT NULL auto_increment,
  `pageId` int(11) NOT NULL,
  `timestamp` int(11) NOT NULL,
  `html` longblob NOT NULL,
  `keep` int(11) NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `pageId` (`pageId`),
  KEY `timeStamp` (`timeStamp`),
  KEY `keep` (`keep`)
) DEFAULT CHARSET=utf8mb4;");
	//var_dump($r);
}

/** Check if config is ok, this is called during install and from cron.
 * @see http://api.drupal.org/api/function/hook_requirements
 * @param phase can be 'install' or 'runtime'. Some checks are different depending on phase.
 * @param the array that will be filled 
 */
function page_watch_config_check($phase,&$req)
{
	global $page_watch_config;

	if(!isset($page_watch_config)){fatal("page_watch_config_check_fast: no config!");}
	// FIXME: should do more checks ?
}


/** @} */

?>